/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.rational.buildforge.buildagent.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.rational.buildforge.buildagent.common.model.IAgentTestResult;
import com.ibm.team.repository.common.TeamRepositoryException;

public interface IBuildAgentConnectClient {

	/**
	 * Provides the ability to request that a connection test to Build Forge
	 * Agent.
	 * 
	 * @param buildAgentProperties
	 *            properties capturing the connection details.
	 * 
	 * @param progressMonitor
	 *            The progress monitor to track progress on or <code>null</code>
	 *            if progress monitoring is not desired.
	 * @return The connection test results.
	 * @throws TeamRepositoryException
	 *             If error occurs when requesting the connection test.
	 * @throws IllegalArgumentException
	 *             If the <code>buildDefinition</code> or any of the slots in
	 *             <code>buildProperties</code> are <code>null</code>.
	 * @LongOp This is a long operation and may block indefinitely. It must not
	 *         be called from a responsive thread.
	 * @deprecated use the other one
	 */
    public IAgentTestResult requestConnectionTest(IBuildProperty[] buildAgentProperties, 
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

    /**
     * Requests a connection test to a Build Forge Agent.
     * 
     * @param buildEngine
     *            The build engine capturing the connection details.
     * @param overrideUserId
     *            A user id that will override the one specified by the
     *            engine, or <code>null</code> for no override.
     * @param overridePassword
     *            An obfuscated password that will override the one specified
     *            by the engine, or <code>null</code> for no override.
     * @param progressMonitor
     *            The progress monitor to track progress on or <code>null</code>
     *            if progress monitoring is not desired.
     * @return The connection test results.
     * @throws TeamRepositoryException
     *             If error occurs when requesting the connection test.
     * @throws IllegalArgumentException
     *             If the <code>buildEngine</code> is <code>null</code>.
     * @LongOp This is a long operation and may block indefinitely. It must not
     *         be called from a responsive thread.
     * @since 0.9 (RTC 4.0)
     */
    public IAgentTestResult requestConnectionTest(IBuildEngine buildEngine, String overrideUserId, String overridePassword,
            IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException;

}

