/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.rational.buildforge.buildagent.internal.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.rational.buildforge.buildagent.client.IBuildAgentConnectClient;
import com.ibm.rational.buildforge.buildagent.common.IBuildAgentConnectService;
import com.ibm.rational.buildforge.buildagent.common.model.IAgentTestResult;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.common.TeamRepositoryException;


public class BuildAgentConnectClient implements IBuildAgentConnectClient{

	private IClientLibraryContext fContext;
	
	/**
     * Creates the abstract team build client for a specified context.
     * 
     * @param context
     *            The context to create the abstract team build client for.
     */
	public BuildAgentConnectClient(IClientLibraryContext context) {
		fContext = context;
	}

    /**
     * Returns IBuildAgentConnectService that implements the given service interface.
     * 
     * @return an object that implements the IBuildAgentConnectService interface; never
     *         <code>null</code>
     */
    private IBuildAgentConnectService getService() throws TeamRepositoryException{
    	IBuildAgentConnectService service = (IBuildAgentConnectService) fContext.getServiceInterface(IBuildAgentConnectService.class);
		if (service == null) {
			throw new TeamRepositoryException(Messages.BuildAgentConnectClient_0);
		}
		return service;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
	public IAgentTestResult requestConnectionTest(
			final IBuildProperty[] buildAgentProperties, IProgressMonitor progressMonitor) throws TeamRepositoryException {
			
		IServiceRunnable<Object> runnable = new IServiceRunnable<Object>() {
			public Object run(IProgressMonitor monitor)
					throws TeamRepositoryException {
				return (IAgentTestResult) getService().requestConnectionTest(buildAgentProperties);
			}
		};
		
		return (IAgentTestResult) fContext.callCancelableService(runnable, progressMonitor);
	}

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
	public IAgentTestResult requestConnectionTest(final IBuildEngine buildEngine, final String overrideUserId, final String overridePassword,
			IProgressMonitor progressMonitor) throws TeamRepositoryException, IllegalArgumentException {

        IServiceRunnable<Object> runnable = new IServiceRunnable<Object>() {
            public Object run(IProgressMonitor monitor)
                    throws TeamRepositoryException {
                return (IAgentTestResult) getService().requestConnectionTest2(buildEngine, overrideUserId, overridePassword);
            }
        };
        
        return (IAgentTestResult) fContext.callCancelableService(runnable, progressMonitor);
	}
}
