/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.rational.connector.buildforge.ui;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.ColumnLayout;
import org.eclipse.ui.forms.widgets.ColumnLayoutData;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.buildforge.services.common.ServiceException;
import com.buildforge.services.common.dbo.MessageDBO;
import com.ibm.rational.connector.buildforge.common.builddefinition.BuildForgeConfigurationData;
import com.ibm.rational.connector.buildforge.common.builddefinition.BuildForgeConfigurationElement;
import com.ibm.rational.connector.buildforge.common.builddefinition.BuildForgeConstants;
import com.ibm.rational.connector.buildforge.internal.client.IBuildForgeServicesLayerClient;
import com.ibm.rational.connector.buildforge.internal.ui.IHelpContextIds;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.internal.ui.editors.builddefinition.BuildDefinitionEditor;
import com.ibm.team.build.ui.editors.builddefinition.AbstractConfigurationElementEditor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;

/**
 * This class handles configuring the Build Forge build definition tab.
 * 
 */
@SuppressWarnings("restriction")
public class BuildForgeConfigurationEditor extends AbstractConfigurationElementEditor {

	private static final int UI_MAX_WIDTH_VALUE = 250;
	
	private IBuildDefinition fBuildDefinitionWorkingCopy;
	private BuildForgeConfigurationData configurationData;
	private Section fBuildForgeProjectSection;
	private Section fBuildForgeLogsSection;
	protected Button fBuildForgeGetProjects;
	protected Button fBuildForgeSynchronizeProperties;
	protected Table fBuildForgeProjectTable;
	protected Button fBuildForgeAllLogs;
	protected Button fBuildForgeAllLogsFrom;
	protected Button fBuildForgeAllLogsNotPassedOrSkipped;
	protected Button fBuildForgeCustomBOM;
	protected Button fBuildForgeNumFirstLogsEnabled;
	protected Button fBuildForgeNumLastLogsEnabled;
	protected Spinner fBuildForgeNumFirstLogs;
	protected Spinner fBuildForgeNumLastLogs;
	private FormToolkit fToolkit;
	private BuildDefinitionEditor fEditor;
	private IBuildDefinition[] currentProjectList = null;
	private Button fBuildForgeCustomLogsEnabled;
	private Text fBuildForgeCustomLogs;
	private Text fBuildForgeProjectFilter;
	private static final String _class = BuildForgeConfigurationEditor.class.getName();
	private boolean fIgnoreFilter = true;

	/**
	 * Create the editor.
	 * 
	 * @param elementId
	 *            The id of the build configuration element being edited.
	 * @param title
	 *            The editor title.
	 */
	public BuildForgeConfigurationEditor(String elementId, String title) {
		super(elementId, title);
	}

	@Override
    public void initialize(BuildDefinitionEditor editor) {
		super.initialize(editor);
		fEditor = editor;
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	@Override
	public void createContent(Composite parent, FormToolkit toolkit) {
		fToolkit = toolkit;
		parent.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB));
		TableWrapLayout bfLayout = new TableWrapLayout();
		bfLayout.numColumns = 2;
		bfLayout.makeColumnsEqualWidth = true;
		bfLayout.horizontalSpacing = 10;
		parent.setLayout(bfLayout);

		fBuildForgeProjectSection = fToolkit.createSection(parent, Section.DESCRIPTION | ExpandableComposite.TITLE_BAR);
		fBuildForgeProjectSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB));
		fBuildForgeProjectSection.setLayout(new TableWrapLayout());
		fBuildForgeProjectSection.setText(BuildForgeMessages.BuildForgeConfigurationEditor_PROJECTS_SECTION_TITLE);
		fBuildForgeProjectSection.setDescription(BuildForgeMessages.BuildForgeConfigurationEditor_PROJECTS_SECTION_DESCRIPTION);

		fBuildForgeLogsSection = fToolkit.createSection(parent, Section.DESCRIPTION | ExpandableComposite.TITLE_BAR);
		fBuildForgeLogsSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB));
		fBuildForgeLogsSection.setLayout(new TableWrapLayout());
		fBuildForgeLogsSection.setText(BuildForgeMessages.BuildForgeConfigurationEditor_LOGS_SECTION_TITLE);
		fBuildForgeLogsSection.setDescription(BuildForgeMessages.BuildForgeConfigurationEditor_LOGS_SECTION_DESCRIPTION);

		Composite projComposite = fToolkit.createComposite(fBuildForgeProjectSection);
		projComposite.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB));
		TableWrapLayout projLayout = new TableWrapLayout();
		projLayout.numColumns = 1;
		projLayout.horizontalSpacing = 10;
		projComposite.setLayout(projLayout);

		createBuildForgeProjectWidgets(projComposite);
		createBuildForgeGetProjectsWidgets(projComposite);

		fBuildForgeProjectSection.setClient(projComposite);

		Composite logsComposite = fToolkit.createComposite(fBuildForgeLogsSection);
		logsComposite.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));
		ColumnLayout logsLayout = new ColumnLayout();
		logsLayout.topMargin = 10;
		logsLayout.maxNumColumns = 1;
		logsLayout.minNumColumns = 1;
		logsComposite.setLayout(logsLayout);
		createBuildForgeLogsWidgets(logsComposite);

		fBuildForgeLogsSection.setClient(logsComposite);
	}

	/**
	 * This creates the widgets for the logs section of the Build Definition.
	 */
	private void createBuildForgeLogsWidgets(Composite parent) {
		boolean customBOM = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_BOM, BuildForgeConstants.DEFAULT_CUSTOM_BOM);
		fBuildForgeCustomBOM = fToolkit.createButton(parent, BuildForgeMessages.BuildForgeConfigurationEditor_INCLUDE_CUSTOM_BOM, SWT.CHECK);
		fBuildForgeCustomBOM.setSelection(customBOM);
		fBuildForgeCustomBOM.addSelectionListener(getBuildForgeCustomBOMSelectionListener());
		fBuildForgeCustomBOM.setSize(SWT.DEFAULT, SWT.DEFAULT);

		/* Create a All Logs composite */
		boolean allLogs = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_ALL_LOGS, BuildForgeConstants.DEFAULT_ALL_LOGS);
		fBuildForgeAllLogs = fToolkit.createButton(parent, BuildForgeMessages.BuildForgeConfigurationEditor_INCLUDE_ALL_LOGS, SWT.RADIO);
		fBuildForgeAllLogs.setSelection(allLogs);
		fBuildForgeAllLogs.addSelectionListener(getBuildForgeAllLogsSelectionListener());
		fBuildForgeAllLogs.setSize(SWT.DEFAULT, SWT.DEFAULT);

		/* Include Step Logs FROM */
		fBuildForgeAllLogsFrom = fToolkit.createButton(parent, BuildForgeMessages.BuildForgeConfigurationEditor_INCLUDE_ALL_LOGS_FROM, SWT.RADIO);
		fBuildForgeAllLogsFrom.setSelection(!allLogs);
		fBuildForgeAllLogsFrom.addSelectionListener(getBuildForgeSpecificLogsFromSelectionListener());
		fBuildForgeAllLogsFrom.setSize(SWT.DEFAULT, SWT.DEFAULT);

		int numFirstLogs = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_FIRST_LOGS, BuildForgeConstants.DEFAULT_FIRST_LOGS);
		boolean enabled = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_FIRST_LOGS_ENABLED, true);
		fBuildForgeNumFirstLogs = createLabeledSpinner(parent, BuildForgeMessages.BuildForgeConfigurationEditor_NUM_FIRST_LOGS_TO_INCLUDE_LABEL,
				BuildForgeMessages.BuildForgeConfigurationEditor_NUM_FIRST_LOGS_TO_INCLUDE_DESC, Integer.toString(numFirstLogs), enabled);
		fBuildForgeNumFirstLogs.addModifyListener(getBuildForgeFirstLogsModifiedListener());

		int numLastLogs = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_LAST_LOGS, BuildForgeConstants.DEFAULT_LAST_LOGS);
		enabled = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_LAST_LOGS_ENABLED, true);
		fBuildForgeNumLastLogs = createLabeledSpinner(parent, BuildForgeMessages.BuildForgeConfigurationEditor_NUM_LAST_LOGS_TO_INCLUDE_LABEL,
				BuildForgeMessages.BuildForgeConfigurationEditor_NUM_LAST_LOGS_TO_INCLUDE_DESC, Integer.toString(numLastLogs), enabled);
		fBuildForgeNumLastLogs.addModifyListener(getBuildForgeLastLogsModifiedListener());

		String customLogs = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_LOGS, "");
		enabled = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_LOGS_ENABLED, false);
		fBuildForgeCustomLogs = createLabeledTextbox(parent, BuildForgeMessages.BuildForgeConfigurationEditor_CUSTOM_LOGS_TO_INCLUDE_LABEL,
				null, customLogs, enabled);
		fBuildForgeCustomLogs.addModifyListener(getBuildForgeCustomLogsModifiedListener());
		
		/* Create a All Logs Not Passed composite */
		boolean notPassed = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_ALL_LOGS_NOT_PASSED_OR_SKIPPED, BuildForgeConstants.DEFAULT_ALL_LOGS_NOT_PASSED_OR_SKIPPED);
		Composite allLogsNotPassedOrSkippedComposite = fToolkit.createComposite(parent);
		ColumnLayout allLogsNotPassedOrSkippedLayout = new ColumnLayout();
		allLogsNotPassedOrSkippedLayout.maxNumColumns = 4;
		allLogsNotPassedOrSkippedLayout.minNumColumns = 1;
		allLogsNotPassedOrSkippedLayout.leftMargin = 20;
		allLogsNotPassedOrSkippedComposite.setLayout(allLogsNotPassedOrSkippedLayout);
		fBuildForgeAllLogsNotPassedOrSkipped = fToolkit.createButton(allLogsNotPassedOrSkippedComposite, BuildForgeMessages.BuildForgeConfigurationEditor_INCLUDE_ALL_LOGS_NOT_PASSED_OR_SKIPPED,
				SWT.CHECK);
		fBuildForgeAllLogsNotPassedOrSkipped.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));
		fBuildForgeAllLogsNotPassedOrSkipped.setSelection(notPassed);
		fBuildForgeAllLogsNotPassedOrSkipped.addSelectionListener(getBuildForgeAllLogsNotPassedOrSkippedSelectionListener());

		updateLogsEnablement();
	}
	
	/**
	 * Creates a listener to check if the custom logs control has been modified.
	 * 
	 * @return the CustomLogs modification listener.
	 */
	private ModifyListener getBuildForgeCustomLogsModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				String customText = fBuildForgeCustomLogs.getText();
				if (customText.matches("(\\s*\\d+\\s*(-\\s*\\d+\\s*)?,?)*")) {
					//valid
					removeMessage(fBuildForgeCustomLogs, fBuildForgeCustomLogs);
				} else {
					//not valid
					addErrorMessage(fBuildForgeCustomLogs, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_CUSTOM_LOG_RANGE, fBuildForgeCustomLogs);
				}
				
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_LOGS, fBuildForgeCustomLogs.getText());
				setDirty(true);
			}
		};
	}
	
	private void updateLogsEnablement() {
		boolean allLogs = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_ALL_LOGS, true);
		boolean firstLogsEnabled = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_FIRST_LOGS_ENABLED, true);
		boolean lastLogsEnabled = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_LAST_LOGS_ENABLED, true);
		boolean customLogsEnabled = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_LOGS_ENABLED, false);

		fBuildForgeAllLogsNotPassedOrSkipped.setEnabled(!allLogs);
		fBuildForgeNumFirstLogsEnabled.setEnabled(!allLogs);
		fBuildForgeNumFirstLogs.setEnabled(!allLogs && firstLogsEnabled);
		fBuildForgeNumLastLogsEnabled.setEnabled(!allLogs);
		fBuildForgeNumLastLogs.setEnabled(!allLogs && lastLogsEnabled);
		fBuildForgeCustomLogsEnabled.setEnabled(!allLogs);
		fBuildForgeCustomLogs.setEnabled(!allLogs && customLogsEnabled);

	}

	/**
	 * Returns a string BF configuration property if set, or the default value otherwise.
	 */
	private String getConfigProperty(String propertyName, String defaultValue) {
		return fBuildDefinitionWorkingCopy.getConfigurationPropertyValue(BuildForgeConfigurationElement.ELEMENT_ID, propertyName, defaultValue);
	}

	/**
	 * Returns a boolean BF configuration property if set, or the default value otherwise.
	 */
	private boolean getConfigProperty(String propertyName, boolean defaultValue) {
		String propValue = getConfigProperty(propertyName, null);
		if (propValue != null && propValue.length() > 0) {
			return Boolean.parseBoolean(propValue);
		}
		return defaultValue;
	}

	/**
	 * Returns an integer BF configuration property if set, or the default value otherwise.
	 */
	private int getConfigProperty(String propertyName, int defaultValue) {
		String propValue = getConfigProperty(propertyName, null);
		if (propValue != null && propValue.length() > 0) {
			try {
				return Integer.parseInt(propValue);
			} catch (NumberFormatException e) {
				// fall through
			}
		}
		return defaultValue;
	}

	/**
     * Sets a string-valued BF configuration property.
     */
    private void setConfigProperty(String propertyName, String value) {
    	fBuildDefinitionWorkingCopy.setConfigurationProperty(
                BuildForgeConfigurationElement.ELEMENT_ID, propertyName, value);
    }

	/**
     * Sets a boolean-valued BF configuration property.
     */
    private void setConfigProperty(String propertyName, boolean value) {
    	setConfigProperty(propertyName, Boolean.toString(value));
    }

	/**
     * Sets an integer-valued BF configuration property.
     */
    private void setConfigProperty(String propertyName, int value) {
    	setConfigProperty(propertyName, Integer.toString(value));
    }

	/**
	 * Creates a listener to check if the last logs control has been modified.
	 * 
	 * @return the LastLogs modification listener.
	 */
	private ModifyListener getBuildForgeLastLogsModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_LAST_LOGS, fBuildForgeNumLastLogs.getSelection());
				setDirty(true);
			}
		};
	}

	/**
	 * Creates a listener to check if the first logs control has been modified.
	 * 
	 * @return the FirstLogs modification listener.
	 */
	private ModifyListener getBuildForgeFirstLogsModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_FIRST_LOGS, fBuildForgeNumFirstLogs.getSelection());
				setDirty(true);
			}
		};
	}

	/**
	 * This creates a listener to check of the AllLogs selection has been
	 * modified. By default it's disabled.
	 */
	protected SelectionListener getBuildForgeAllLogsSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_ALL_LOGS, true);
				updateLogsEnablement();
				setDirty(true);
			}
		};
	}

	/**
	 * This creates a listener to check of the SpecificLogs (allows you to
	 * configure first/last logs) selection has been modified. By default it's
	 * enabled.
	 */
	protected SelectionListener getBuildForgeSpecificLogsFromSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_ALL_LOGS, false);
				updateLogsEnablement();
				setDirty(true);
			}
		};
	}

	/**
	 * This creates a listener to check if "AllLogsNotPassedOrSkipped" is
	 * selected. This control allows for steps marked warn or fail to pull in
	 * those logs which are likely most important for troubleshooting.
	 */
	protected SelectionListener getBuildForgeAllLogsNotPassedOrSkippedSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_ALL_LOGS_NOT_PASSED_OR_SKIPPED, fBuildForgeAllLogsNotPassedOrSkipped.getSelection());
				setDirty(true);
			}
		};
	}

	/**
	 * This creates a listener to determine if the BOM info should be pulled in
	 * or not.
	 */
	protected SelectionListener getBuildForgeCustomBOMSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_BOM, fBuildForgeCustomBOM.getSelection());
				setDirty(true);
			}
		};
	}

	/**
	 * Creates the push button for "Get Projects" on the Build Forge tab.
	 */
	private void createBuildForgeGetProjectsWidgets(Composite parent) {
		createSpacer(parent, 5, 1);
		
		fBuildForgeGetProjects = fToolkit.createButton(parent, BuildForgeMessages.BuildForgeConfigurationEditor_GET_PROJECTS_BUTTON, SWT.PUSH);
		fBuildForgeGetProjects.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_GET_PROJECTS_BUTTON);
		fBuildForgeGetProjects.setText(BuildForgeMessages.BuildForgeConfigurationEditor_GET_PROJECTS_BUTTON);
		fBuildForgeGetProjects.addSelectionListener(getBuildForgeGetProjectsSelectionListener());

		Label getProjectsLabel = fToolkit.createLabel(parent, BuildForgeMessages.BuildForgeConfigurationEditor_GET_PROJECTS_BUTTON_DESC, SWT.WRAP);
		TableWrapData getLabelData = new TableWrapData(TableWrapData.FILL);
		getLabelData.maxWidth = UI_MAX_WIDTH_VALUE;
		getProjectsLabel.setLayoutData(getLabelData);
		getProjectsLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GRAY));

		createSpacer(parent, 5, 1);
		
		/* Add a Synchronize Properties push button. */
		fBuildForgeSynchronizeProperties = fToolkit.createButton(parent, BuildForgeMessages.BuildForgeConfigurationEditor_SYNC_PROPERTIES_BUTTON, SWT.PUSH);
		fBuildForgeSynchronizeProperties.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_SYNC_PROPERTIES_BUTTON);
		fBuildForgeSynchronizeProperties.setText(BuildForgeMessages.BuildForgeConfigurationEditor_SYNC_PROPERTIES_BUTTON);
		fBuildForgeSynchronizeProperties.addSelectionListener(getBuildForgeSynchronizePropertiesSelectionListener());

		Label syncPropertiesLabel = fToolkit.createLabel(parent, BuildForgeMessages.BuildForgeConfigurationEditor_SYNC_PROPERTIES_BUTTON_DESC, SWT.WRAP);
		TableWrapData syncLabelData = new TableWrapData(TableWrapData.FILL);
		syncLabelData.maxWidth = UI_MAX_WIDTH_VALUE;
		syncPropertiesLabel.setLayoutData(syncLabelData);
		syncPropertiesLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GRAY));
	}

	/**
	 * Creates a listener to alert if the "Get Projects" button has been pushed.
	 */
	protected SelectionListener getBuildForgeGetProjectsSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (validateBuildForgeEngines()) {
					if (fBuildForgeProjectTable.getItemCount() > 0) {
						fBuildForgeProjectTable.removeAll();
					}
					if (!fBuildForgeProjectFilter.getEnabled()) {
						fBuildForgeProjectFilter.setEnabled(true);
					}
					/* Getting a new list of projects again should clear all of these errors. */
					removeMessage(fBuildForgeProjectTable, fBuildForgeProjectTable);
					removeMessage(fBuildForgeGetProjects, fBuildForgeGetProjects);
					removeMessage(fBuildForgeSynchronizeProperties, fBuildForgeSynchronizeProperties);
					getProjects();
				}
			}
		};
	}

	/**
	 * Creates a listener to alert if the "Synchronize Properties" button has been pushed.
	 */
	protected SelectionListener getBuildForgeSynchronizePropertiesSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				if (validateBuildForgeEngines()) {
					removeMessage(fBuildForgeSynchronizeProperties, fBuildForgeSynchronizeProperties);
					IBuildForgeServicesLayerClient servicesLayerClient = 
						(IBuildForgeServicesLayerClient) getTeamRepository().getClientLibrary(IBuildForgeServicesLayerClient.class);
					
					try {
						IBuildDefinition definition = servicesLayerClient.synchronizeProperties((IBuildEngine[])fEditor.getSupportingEngines().toArray(new IBuildEngine[fEditor.getSupportingEngines().size()]), fBuildDefinitionWorkingCopy, null, null);
						updateWorkingCopyProperties (definition);
					} catch (TeamRepositoryException tre) {
						/* TODO:  Need to fix this message up so we're not appending the exception text. */
						addErrorMessage(fBuildForgeSynchronizeProperties, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_GETTING_BUILDFORGE_ENVIRONMENT + "  " + getDisplayTextFromException(tre), fBuildForgeSynchronizeProperties); //$NON-NLS-1$
						BuildForgePlugin.log(_class, tre);
					} finally {
						setDirty(true);
					}
				}
			}
		};
	}
	
	/**
	 * @return
	 * 
	 * Validate that the Build Forge engines associated with the editor are valid and refer to the same BF database.
	 */
	protected boolean validateBuildForgeEngines() {
		boolean isExists = validateBuildForgeEngineExists();
		boolean isMatch = validateBuildForgeEngineMatch();
		if (isExists && isMatch) {
			removeMessage(fBuildForgeProjectTable, fBuildForgeProjectTable);
			return true;
		} else {
			if (!isExists) {
				addErrorMessage(fBuildForgeProjectTable, BuildForgeMessages.BuildForgeConfigurationEditor_ENGINE_NONEXISTENT, fBuildForgeProjectTable);
			}
			if (!isMatch) {
				addErrorMessage(fBuildForgeProjectTable, BuildForgeMessages.BuildForgeConfigurationEditor_ENGINE_MISMATCH, fBuildForgeProjectTable);
			}
			return false;
		}
	}

	
	/**
	 * Check all the engines in the Build Definition to see if they refer to the same Build Forge console.  This is done by
	 * creating a master list of all the projects for the first engine, and then iterating through the rest of the build 
	 * engines on the build definition, checking the projects lists against the master to ensure that they match.
	 * 
	 * @return
	 * whether or not the engines all point to the same Build Forge console
	 */
	private boolean validateBuildForgeEngineMatch() {
		List<IBuildEngine> engines = fEditor.getSupportingEngines();
		IBuildDefinition[] masterList = null;
		if (engines.size() > 1) {
			//if there are more than one engine, we need to start comparing.
			try {
				for (IBuildEngine engine : engines) {
					if (engine.getConfigurationElement(BuildForgeConfigurationElement.ENGINE_ELEMENT_ID) != null) {
						IBuildForgeServicesLayerClient servicesLayerClient = 
							(IBuildForgeServicesLayerClient) getTeamRepository().getClientLibrary(IBuildForgeServicesLayerClient.class);		
						if (masterList == null) {
							//First iteration, set up master list of projects
							masterList = servicesLayerClient.getProjects(engine, null);
							continue;
						}
						IBuildDefinition[] matchList = servicesLayerClient.getProjects(engine, null);
						//If the lists are different lengths look no further!
						if (masterList.length != matchList.length) {
							return false;
						}
						//Match up the project uuids.  If one is NOT found then return false! 
						for (IBuildDefinition match : matchList) {
							boolean isFound = false;
							for (IBuildDefinition original : masterList) {
								if (original.getPropertyValue(BuildForgeConstants.PROJECT_UUID_PROPERTY, "").equals(match.getPropertyValue(BuildForgeConstants.PROJECT_UUID_PROPERTY, ""))) {
									//Found project
									isFound = true;
									break;
								}
							}
							if (!isFound) {
								return false;
							}
						}	
					}
				}
			} catch (TeamRepositoryException e) {
				addErrorMessage(fBuildForgeGetProjects, NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_GETTING_PROJECTS_FROM_BUILDFORGE, getDisplayTextFromException(e)), fBuildForgeGetProjects);
				BuildForgePlugin.log(_class, e);
			}
		}
		return true;
	}

	/**
	 * This method checks the TRE for different exception text to display.
	 */
	private String getDisplayTextFromException (TeamRepositoryException tre) {
		String message = tre.getMessage() != null ? tre.getMessage() : tre.getClass().getName();
		Throwable cause = tre.getCause();
		if (cause != null) {
			if (cause instanceof ServiceException) {
				MessageDBO messageDBO = ((ServiceException)cause).getMessageObject();
				message = messageDBO.getMessageKey();
			} else {
				message = cause.getMessage() != null ? cause.getMessage() : cause.getClass().getName();
			}
		} 
		return message;
	}
	/**
	 * This method clears the working copy properties and resets 
	 * them from the definition passed back from synchronizeProperties.
	 */
	@SuppressWarnings({ "unchecked", "rawtypes" })
	private void updateWorkingCopyProperties(IBuildDefinition definition) {
		if (definition != null) {
			fBuildDefinitionWorkingCopy.getProperties().clear();
			Collection updatedPropertyCollection = Collections.unmodifiableList(definition.getProperties());
			fBuildDefinitionWorkingCopy.getProperties().addAll(updatedPropertyCollection);
		} else {
			throw new IllegalArgumentException (BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_SYNCHRONIZING_PROPERTIES_WITH_BUILDFORGE);
		}
	}
	
	/**
	 * Creates the Projects list that shows the Projects queried from Build
	 * Forge.
	 */
	private void createBuildForgeProjectWidgets(Composite parent) {
		String propValue = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PROJECT, ""); //$NON-NLS-1$
		
		createSpacer(parent, 5, 1);
		fBuildForgeProjectFilter = fToolkit.createText(parent, BuildForgeMessages.BuildForgeConfigurationEditor_FILTER_TEXT);
		fBuildForgeProjectFilter.setEnabled(false);
		fBuildForgeProjectFilter.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		fBuildForgeProjectFilter.addFocusListener(getBuildForgeFilterFocusListener());
		fBuildForgeProjectFilter.addKeyListener(getBuildForgeFilterKeyListener());
		fBuildForgeProjectTable = fToolkit.createTable(parent, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		TableWrapData layoutData = new TableWrapData(TableWrapData.FILL_GRAB);
		layoutData.heightHint = fBuildForgeProjectTable.getItemHeight() * 10;
		fBuildForgeProjectTable.setLayoutData(layoutData);

		if (!propValue.trim().equals("")) { //$NON-NLS-1$
			TableItem tableItem = new TableItem(fBuildForgeProjectTable, 0);
			tableItem.setText(propValue);
			fBuildForgeProjectTable.setSelection(0);
		}

		fBuildForgeProjectTable.addSelectionListener(getBuildForgeProjectsSelectionListener());
		Label descriptionLabel = fToolkit.createLabel(parent, BuildForgeMessages.BuildForgeConfigurationEditor_PROJECTS_DESC, SWT.WRAP);
		descriptionLabel.setLayoutData(new TableWrapData(TableWrapData.FILL));
		descriptionLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GRAY));
	}

	/**
	 * Creates a listener to alert when a Project has been selected.
	 * 
	 * @return a SelectionListener
	 */
	protected SelectionListener getBuildForgeProjectsSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					String currentProjectUUID = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PROJECT_UUID, null);
					if (validate()) {
						TableItem[] tableItem = fBuildForgeProjectTable.getSelection();
						String selectedItemText = ""; //$NON-NLS-1$
						for (int i = 0; i < tableItem.length; i++) {
							selectedItemText = tableItem[i].getText();
						}
						setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PROJECT, selectedItemText);
						
						/* Look for project from list based on name. */
						String projectUuid = ""; //$NON-NLS-1$
						for (IBuildDefinition project : currentProjectList) {
							String projectName = project.getId().trim();
							if (projectName.equals(selectedItemText)) {
								projectUuid = project.getProperty(BuildForgeConstants.PROJECT_UUID_PROPERTY).getValue();
							}
						}
						setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PROJECT_UUID, projectUuid);
						
						/* Call to service to sync properties with Build Forge */
						IBuildForgeServicesLayerClient servicesLayerClient = 
							(IBuildForgeServicesLayerClient) getTeamRepository().getClientLibrary(IBuildForgeServicesLayerClient.class);
						
						try {
							IBuildDefinition definition = servicesLayerClient.synchronizeProperties((IBuildEngine[])fEditor.getSupportingEngines().toArray(new IBuildEngine[fEditor.getSupportingEngines().size()]), fBuildDefinitionWorkingCopy.copyDefinition(), currentProjectUUID, null);
							updateWorkingCopyProperties (definition);
						} catch (TeamRepositoryException tre) {
							BuildForgePlugin.log(_class, tre);
						} finally {
							setDirty(true);
						}
					}
				} catch (Exception t) {
					BuildForgePlugin.log(_class, t);
					addErrorMessage(fBuildForgeGetProjects, NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_GETTING_BUILDFORGE_ENVIRONMENT, t), fBuildForgeGetProjects);
				}
			}
		};
	}

	/**
	 * Gets the SelectionListener for the FirstLogsEnabled flag.
	 * 
	 * @return a created SelectionListener
	 */
	protected SelectionListener getBuildForgeNumFirstLogsEnabledListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean selected = fBuildForgeNumFirstLogsEnabled.getSelection();
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_FIRST_LOGS_ENABLED, selected);
				updateLogsEnablement();
				setDirty(true);
			}
		};
	}

	/**
	 * Gets the SelectionListener for the LastLogsEnabled flag.
	 * 
	 * @return a created SelectionListener
	 */
	protected SelectionListener getBuildForgeNumLastLogsEnabledListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean selected = fBuildForgeNumLastLogsEnabled.getSelection();
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_LAST_LOGS_ENABLED, selected);
				updateLogsEnablement();
				setDirty(true);
			}
		};
	}

	/**
	 * Creates a text control with specified label for use in the logs
	 * section of the build definition.
	 * 
	 * @return the Text control after configured
	 */
	private Text createLabeledTextbox(Composite parent, String label, String description, String textValue, boolean enabled) {
		Composite labeledTextboxComposite = fToolkit.createComposite(parent);
		ColumnLayout labeledTextboxLayout = new ColumnLayout();
		labeledTextboxLayout.maxNumColumns = 6;
		labeledTextboxLayout.minNumColumns = 3;
		labeledTextboxLayout.horizontalSpacing = 2;
		labeledTextboxLayout.leftMargin = 20;
		labeledTextboxComposite.setLayout(labeledTextboxLayout);
		
		fBuildForgeCustomLogsEnabled = fToolkit.createButton(labeledTextboxComposite, label, SWT.CHECK);
		fBuildForgeCustomLogsEnabled.setSelection(enabled);
		fBuildForgeCustomLogsEnabled.addSelectionListener(getBuildForgeCustomLogsEnabledListener());

		final Text textBox = new Text(labeledTextboxComposite, SWT.BORDER);
		textBox.setText(textValue);
		textBox.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_CUSTOM_LOGS_TOOLTIP);
		textBox.setEnabled(fBuildForgeCustomLogsEnabled.getSelection());
		
		Label descLabel = fToolkit.createLabel(labeledTextboxComposite, description);
		descLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));
		
		Label spaceLabel = fToolkit.createLabel(labeledTextboxComposite, "  "); //$NON-NLS-1$
		spaceLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));

		spaceLabel = fToolkit.createLabel(labeledTextboxComposite, "  "); //$NON-NLS-1$
		spaceLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));

		spaceLabel = fToolkit.createLabel(labeledTextboxComposite, "  "); //$NON-NLS-1$
		spaceLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));
		
		return textBox;
	}

	
	private SelectionListener getBuildForgeCustomLogsEnabledListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean selected = fBuildForgeCustomLogsEnabled.getSelection();
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_CUSTOM_LOGS_ENABLED, selected);
				updateLogsEnablement();
				setDirty(true);
			}
		};
	}
	
	/**
	 * Creates and returns a FocusListener for the filter text box.
	 * When focus is gained, ignoreFilter flag is set to false and the 
	 * text of the filter is set to an empty string. When focus is lost, 
	 * if the contents of the filter is an empty string or null, the 
	 * ignoreFilter flag is set to true and the filter text is set to 
	 * default filter.
	 *
	 * @return the FocusListener initialize and blank out the filter text when the filter box focus changes
	 */
	private FocusListener getBuildForgeFilterFocusListener() {
		return new FocusListener(){
			@Override
			public void focusGained(FocusEvent e) {
				if (fIgnoreFilter) {
					fIgnoreFilter = false;
					fBuildForgeProjectFilter.setText("");
				}
			}

			@Override
			public void focusLost(FocusEvent e) {
				if (fBuildForgeProjectFilter.getText() == null || fBuildForgeProjectFilter.getText().isEmpty()) {
					fIgnoreFilter = true;
					fBuildForgeProjectFilter.setText(BuildForgeMessages.BuildForgeConfigurationEditor_FILTER_TEXT);
				}
			}
		};
	}
	
	/**
	 * Creates and returns a KeyListener for the BuildForge project filter.
	 * When a key is released, the project table is built again, applying the filter. 
	 *
	 * @return the KeyListener for the filter box.
	 */
	private KeyListener getBuildForgeFilterKeyListener() {
		return new KeyListener() {
			@Override
			public void keyPressed(KeyEvent e) {
				// Does nothing
			}

			@Override
			public void keyReleased(KeyEvent e) {
				fBuildForgeProjectTable.removeAll();
				buildProjectTable();
			}	
		};
	}
	
	/**
	 * Creates a spinner control with specified label for use in the logs
	 * section of the build definition.
	 * 
	 * @return the Spinner control after configured
	 */
	private Spinner createLabeledSpinner(Composite parent, String label, String description, String textValue, boolean enabled) {
		Composite spinnerComposite = fToolkit.createComposite(parent);
		ColumnLayout spinnerLayout = new ColumnLayout();
		spinnerLayout.maxNumColumns = 6;
		spinnerLayout.minNumColumns = 3;
		spinnerLayout.horizontalSpacing = 2;
		spinnerLayout.leftMargin = 20;
		spinnerComposite.setLayout(spinnerLayout);

		// Check box
		if (label.equals(BuildForgeMessages.BuildForgeConfigurationEditor_NUM_FIRST_LOGS_TO_INCLUDE_LABEL)) {
			fBuildForgeNumFirstLogsEnabled = fToolkit.createButton(spinnerComposite, label, SWT.CHECK);
			fBuildForgeNumFirstLogsEnabled.setLayoutData(new ColumnLayoutData(ColumnLayoutData.CENTER));
			fBuildForgeNumFirstLogsEnabled.setSelection(enabled);
			fBuildForgeNumFirstLogsEnabled.addSelectionListener(getBuildForgeNumFirstLogsEnabledListener());
		} else {
			fBuildForgeNumLastLogsEnabled = fToolkit.createButton(spinnerComposite, label, SWT.CHECK);
			fBuildForgeNumLastLogsEnabled.setLayoutData(new ColumnLayoutData(ColumnLayoutData.CENTER));
			fBuildForgeNumLastLogsEnabled.setSelection(enabled);
			fBuildForgeNumLastLogsEnabled.addSelectionListener(getBuildForgeNumLastLogsEnabledListener());
		}

		final Spinner spinner = new Spinner(spinnerComposite, SWT.BORDER);
		spinner.setLayoutData(new ColumnLayoutData(ColumnLayoutData.CENTER));
		spinner.setValues(Integer.valueOf(textValue).intValue(), 0, 100, 0, 1, 5);
		Point size = spinner.computeSize(20, SWT.DEFAULT);
		spinner.setSize(size.x / 3, size.y);
		spinner.setEnabled(enabled);

		// Label
		Label lastLabel = fToolkit.createLabel(spinnerComposite, description);
		lastLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));

		Label spaceLabel = fToolkit.createLabel(spinnerComposite, "  "); //$NON-NLS-1$
		spaceLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));

		spaceLabel = fToolkit.createLabel(spinnerComposite, "  "); //$NON-NLS-1$
		spaceLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));

		spaceLabel = fToolkit.createLabel(spinnerComposite, "  "); //$NON-NLS-1$
		spaceLabel.setLayoutData(new ColumnLayoutData(ColumnLayoutData.LEFT));

		return spinner;
	}

	/**
	 * Creates a spacer for the UI dialog within the Composite.
	 */
	private void createSpacer(Composite parent, int height, int horizontalSpan) {
		Label label = fToolkit.createLabel(parent, ""); //$NON-NLS-1$
		TableWrapData tableWrapData = new TableWrapData(TableWrapData.FILL, TableWrapData.MIDDLE, 1, horizontalSpan);
		tableWrapData.heightHint = height;
		label.setLayoutData(tableWrapData);
	}

	/**
	 * Retrieves the list of Build Forge Projects from the Build Forge Server -
	 * because we are using the Services Java Client, this list has been limited
	 * by the permissions of the user specified.
	 */
	private void getProjects() {
		BuildForgePlugin.log(_class, "getProjects: " + configurationData.toString()); //$NON-NLS-1$
		
		IBuildForgeServicesLayerClient servicesLayerClient = 
			(IBuildForgeServicesLayerClient) getTeamRepository().getClientLibrary(IBuildForgeServicesLayerClient.class);
		
		try {
			currentProjectList = servicesLayerClient.getProjects(getFirstBuildForgeEngine(), null);
		} catch (TeamRepositoryException e) {
			addErrorMessage(fBuildForgeGetProjects, NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_GETTING_PROJECTS_FROM_BUILDFORGE, getDisplayTextFromException(e)), fBuildForgeGetProjects);
			BuildForgePlugin.log(_class, e);
		}
		
		buildProjectTable();
	}
	
	/*
	 * Builds the project table, applying the filter if required.
	 */
	private void buildProjectTable() {
		/* This starts with an empty list of Projects.  It fills the list and sets the selection,
		 * if that project still exists in the list. */
		if (currentProjectList != null && currentProjectList.length > 0) {
			for (int i = 0; i < currentProjectList.length; i++) {
				IBuildDefinition project = currentProjectList[i];
				if (project != null) {
					String projectName = project.getId();
					if (fIgnoreFilter || projectName.toLowerCase().contains(fBuildForgeProjectFilter.getText().toLowerCase())) {
						boolean isLibrary = Boolean.valueOf(project.getProperty(BuildForgeConstants.PROJECT_LIBRARY_PROPERTY).getValue());
						boolean isDefaultSnapshot = Boolean.valueOf(project.getProperty(BuildForgeConstants.PROJECT_DEFAULT_SNAPSHOT_PROPERTY).getValue());
						boolean isActive = Boolean.valueOf(project.getProperty(BuildForgeConstants.PROJECT_ACTIVE_PROPERTY).getValue());
						if (!isLibrary && isDefaultSnapshot && isActive) {
							TableItem tableItem = new TableItem(fBuildForgeProjectTable, 0);
							tableItem.setText(projectName);
							String configurationProjectName = configurationData.getProjectName();
							if (configurationProjectName != null && configurationProjectName.equals(projectName)) {
								fBuildForgeProjectTable.setSelection(i);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * This will display an error dialog if not build engine exists.
	 */
	private boolean validateBuildForgeEngineExists() {
		IBuildEngine buildEngine = getFirstBuildForgeEngine();
		if (buildEngine == null) {
			MessageDialog.openError(getSite().getShell(), BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_NO_BUILD_ENGINE_TITLE, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_MISSING_BUILD_ENGINE);
			return false;
		}
		return true;
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see com.ibm.team.build.ui.editors.builddefinition.
	 * AbstractConfigurationElementEditor#validate()
	 */
	@Override
	public boolean validate() {
		return validateProject() && validateBuildForgeEngines();
	}

	/**
	 * Validates that the Project is specified when creating a new "Build
	 * Definition" or when editing a "Build Definition."
	 * 
	 * @return if validation is successful
	 */
	private boolean validateProject() {
		boolean isValid = true;

		TableItem[] tableItem = fBuildForgeProjectTable.getSelection();
		String selectedItemText = ""; //$NON-NLS-1$
		for (int i = 0; i < tableItem.length; i++) {
			selectedItemText = tableItem[i].getText();
		}

		if (selectedItemText.trim().equals("")) { //$NON-NLS-1$
			addErrorMessage(fBuildForgeProjectTable, BuildForgeMessages.BuildForgeConfigurationEditor_PROJECT_REQUIRED, fBuildForgeProjectTable);
			isValid = false;
		} else {
			removeMessage(fBuildForgeProjectTable, fBuildForgeProjectTable);
		}
		setPageStatusIndicator(!isValid, false);		
		return isValid;
	}

	/**
	 * Returns the first BF build engine, or <code>null</code> if none selected.
	 */
	private IBuildEngine getFirstBuildForgeEngine() {
		List<IBuildEngine> engines = fEditor.getSupportingEngines();
		if (!engines.isEmpty()) {
			for (IBuildEngine engine : engines) {
				if (engine.getConfigurationElement(BuildForgeConfigurationElement.ENGINE_ELEMENT_ID) != null) {
					return engine;
				}
			}
		}
		return null;
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	@Override
	public void setWorkingCopy(IBuildDefinition definition) {
		configurationData = new BuildForgeConfigurationData(definition);
		fBuildDefinitionWorkingCopy = definition;
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	@Override
	public Control getFocusControl() {
		return fBuildForgeProjectTable;
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	@Override
	protected String getContextHelpId() {
		return IHelpContextIds.HELP_CONTEXT_BUILD_DEFINITION_EDITOR_BUILD_FORGE;
	}
	
}
