/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.rational.connector.buildforge.ui;

import java.security.GeneralSecurityException;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import com.ibm.rational.connector.buildforge.common.builddefinition.BuildForgeConfigurationElement;
import com.ibm.rational.connector.buildforge.common.builddefinition.BuildForgeConstants;
import com.ibm.rational.connector.buildforge.common.builddefinition.BuildForgeEngineConfigurationData;
import com.ibm.rational.connector.buildforge.internal.client.IBuildForgeServicesLayerClient;
import com.ibm.rational.connector.buildforge.internal.ui.IHelpContextIds;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.ui.editors.buildengine.BasicEngineConfigurationElementEditor;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.util.NLS;
import com.ibm.team.repository.common.util.ObfuscationHelper;

/**
 * This class handles configuring the Build Forge build engine tab.
 * 
 */
public class BuildForgeEngineConfigurationEditor extends BasicEngineConfigurationElementEditor {

    private static final int SPACER_HEIGHT = 10;
	
	private FormToolkit fToolkit;
	protected Text fBuildForgeHost;
	protected Text fBuildForgePort;
	protected Text fBuildForgeUserId;
	protected Text fBuildForgePassword;
	protected Button fBuildForgeConnectSecure;
    protected Button fBuildForgeTestConnection;
    protected Text fBuildForgeConnectionText;
    private static final String _class = BuildForgeEngineConfigurationEditor.class.getName();	

	/**
	 * Create the editor.
	 * 
	 * @param elementId
	 *            The id of the build engine configuration element being edited.
	 * @param title
	 *            The editor title.
	 */
	public BuildForgeEngineConfigurationEditor(String elementId, String title) {
		super(elementId, title);
	}

	@Override
	public void createContent(Composite parent, FormToolkit toolkit) {
		fToolkit = toolkit;
		TableWrapLayout bfLayout = new TableWrapLayout();
		bfLayout.numColumns = 2;
		bfLayout.makeColumnsEqualWidth = true;
		bfLayout.horizontalSpacing = 10;
		parent.setLayout(bfLayout);

		// server section
        Section serverSection = createSection(parent,
        		BuildForgeMessages.BuildForgeConfigurationEditor_ENGINE_SECTION_TITLE,
        		BuildForgeMessages.BuildForgeConfigurationEditor_ENGINE_SECTION_DESCRIPTION, false);
        Composite serverComposite = (Composite) serverSection.getClient();
        serverSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB, 1, 1));

		createBuildForgeSecureConnectWidgets(serverComposite);
		createSpacer(serverComposite, SPACER_HEIGHT, 2);
		createBuildForgeHostNameWidgets(serverComposite);
		createBuildForgePortWidgets(serverComposite);
		createBuildForgeUserIdWidgets(serverComposite);
		createBuildForgePasswordWidgets(serverComposite);
        
        // test connection section
        Section testConnectionSection = createSection(parent,
        		BuildForgeMessages.BuildForgeConfigurationEditor_TEST_CONNECTION_SECTION_TITLE,
        		BuildForgeMessages.BuildForgeConfigurationEditor_TEST_CONNECTION_SECTION_DESCRIPTION, false, 1);
        Composite testConnectionComposite = (Composite) testConnectionSection.getClient();
        testConnectionSection.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL, 1, 1));
        
        createBuildForgeTestResultsWidgets(testConnectionComposite);      
        createBuildForgeTestConnectionWidgets(testConnectionComposite);
	}

    private void createBuildForgeTestResultsWidgets(Composite parent) {
       	fBuildForgeConnectionText = fToolkit.createText(parent, null, SWT.READ_ONLY | SWT.NO_FOCUS | SWT.MULTI | SWT.V_SCROLL  | SWT.WRAP);
       	fBuildForgeConnectionText.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.FILL_GRAB, 40,1 ));
       }
	
    private void createBuildForgeTestConnectionWidgets(Composite parent){
    	fBuildForgeTestConnection = fToolkit.createButton(parent,
                BuildForgeMessages.BuildForgeConfigurationEditor_TEST_CONNECTION_BUTTON, SWT.PUSH);
    	fBuildForgeTestConnection.addSelectionListener(getBuildForgeTestConnectionSelectionListener());
    }
    
    protected SelectionListener getBuildForgeTestConnectionSelectionListener() {
        return new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	removeMessage(fBuildForgeTestConnection, fBuildForgeTestConnection);
				testConnection();
            }
    
       };
    }
	
    /*
     * Tests Agent Connection
     */
	private void testConnection() {
		
		fBuildForgeTestConnection.setEnabled(false);

    	removeMessage(fBuildForgeTestConnection, fBuildForgeTestConnection);
		fBuildForgeConnectionText.setText(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_TESTING + "\n"); //$NON-NLS-1$
		
		Job job = new Job(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST) { 
			protected IStatus run(IProgressMonitor monitor) {
				try {
					IBuildForgeServicesLayerClient servicesLayerClient = 
						(IBuildForgeServicesLayerClient) getTeamRepository().getClientLibrary(IBuildForgeServicesLayerClient.class);
					IBuildProperty[] resultProperties = null;
					String resultMessage = "\t" + BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_RESULT; //$NON-NLS-1$
					try {
						resultProperties = servicesLayerClient.requestConnectionTest2(fWorkingCopy, monitor);
					} catch (TeamRepositoryException e) {
						BuildForgePlugin.log(_class, e);
						resultMessage = resultMessage + e.getClass().getName() + " - " + e.getMessage();  //$NON-NLS-1$
					} catch (IllegalArgumentException iae) {
						BuildForgePlugin.log(_class, iae);
						resultMessage = resultMessage + iae.getClass().getName() + " - " + iae.getMessage();  //$NON-NLS-1$
					}
					
					final Map<String,String> resultMap = BuildForgeEngineConfigurationData.convertResultPropertiesToMap(resultProperties);
					final StringBuilder message = new StringBuilder(resultMessage);
					if (resultMap != null) {
						if (BuildForgePlugin.isDebugEnabled()) {
						    BuildForgePlugin.log(_class, "Result properties: " + resultMap); //$NON-NLS-1$
						}
						String status = resultMap.get(BuildForgeConstants.TEST_RESULT_STATUS_PROPERTY);
						if (status != null && status.equals(Boolean.TRUE.toString())) {
							message.append(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_SUCCESS);
							message.append("\n\t").append(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_PING).append(" ").append(resultMap.get(BuildForgeConstants.TEST_RESULT_PING_PROPERTY));  //$NON-NLS-1$//$NON-NLS-2$
							message.append("\n\t").append(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_LOGIN).append(" ").append(resultMap.get(BuildForgeConstants.TEST_RESULT_USERID_PROPERTY)); //$NON-NLS-1$ //$NON-NLS-2$
							message.append("\n\t").append(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_NAME).append(" ").append(resultMap.get(BuildForgeConstants.TEST_RESULT_USER_NAME_PROPERTY));  //$NON-NLS-1$//$NON-NLS-2$
						} else {
							String testExceptionType = resultMap.get(BuildForgeConstants.TEST_RESULT_EXCEPTION_TYPE_PROPERTY);
							String testExceptionName = resultMap.get(BuildForgeConstants.TEST_RESULT_EXCEPTION_NAME_PROPERTY);
							String testExceptionMessage = resultMap.get(BuildForgeConstants.TEST_RESULT_EXCEPTION_MESSAGE_PROPERTY);
							if (testExceptionType != null && testExceptionName != null) {
								if (testExceptionType.equals(BuildForgeConstants.TEST_RESULT_SERVICE_EXCEPTION_TYPE)) {
									message.append(" ").append(NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_SERVICE_EXCEPTION, testExceptionName)); //$NON-NLS-1$
								} else if(testExceptionType.equals(BuildForgeConstants.TEST_RESULT_SSL_SERVICE_EXCEPTION_TYPE)) {
									message.append(" ").append(NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_SSL_SERVICE_EXCEPTION, testExceptionName)); //$NON-NLS-1$
								} else if (testExceptionType.equals(BuildForgeConstants.TEST_RESULT_IO_EXCEPTION_TYPE)) {
									message.append(" ").append(NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_IO_EXCEPTION, testExceptionName));									 //$NON-NLS-1$
								} else {
									message.append(" ").append(NLS.bind(BuildForgeMessages.BuildForgeConfigurationEditor_CONNECTION_TEST_TEXT_GENERIC_EXCEPTION, testExceptionName)); //$NON-NLS-1$
								}
							}
							if (testExceptionMessage != null) {
								message.append("\n\t").append(testExceptionMessage); //$NON-NLS-1$
							}
		                    Display.getDefault().syncExec(new Runnable() {
		                        public void run() {
		                            if (!fBuildForgeTestConnection.isDisposed()) {
		    							addWarningMessage(fBuildForgeTestConnection, BuildForgeMessages.BuildForgeConfigurationEditor_TEST_CONNECTION_WARNING, fBuildForgeTestConnection);
		                            }
		                        }
		                    });
						}
					}
					appendTestConnectionText(message.toString());
                } finally {

                    Display.getDefault().syncExec(new Runnable() {
                        public void run() {
                            if (!fBuildForgeTestConnection.isDisposed()) {
                            	fBuildForgeTestConnection.setEnabled(true);
                            }
                        }
                    });
                }
				return Status.OK_STATUS;
			}

            private void appendTestConnectionText(final String text) {
                if (text != null) {
                    Display.getDefault().syncExec(new Runnable() {
                        public void run() {
                            if (!fBuildForgeConnectionText.isDisposed()) {
                            	fBuildForgeConnectionText.append(text + "\n"); //$NON-NLS-1$
                            }
                        }
                    });
                }
            }
		};
		job.schedule();
    }

	/**
	 * This creates the host name widgets in the Build Forge tab.
	 * 
	 * @param parent
	 */
	private void createBuildForgeHostNameWidgets(Composite parent) {
	    String propValue = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_HOSTNAME, ""); //$NON-NLS-1$

		fBuildForgeHost = createLabeledText(parent, BuildForgeMessages.BuildForgeConfigurationEditor_HOST_NAME_LABEL, null, propValue);
		fBuildForgeHost.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_HOST_NAME_DESC);

		fBuildForgeHost.addModifyListener(getBuildForgeHostNameModifiedListener());
	}

	/**
	 * This determines if the host name has been changed.
	 * 
	 * @return a ModifyListener
	 */
	private ModifyListener getBuildForgeHostNameModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
			    setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_HOSTNAME, fBuildForgeHost.getText().trim());
			    clearPassword();
				setDirty(true);
			    validate();
			}
		};
	}

	/**
	 * This creates the widgets for the port in the Build Forge tab.
	 * 
	 * @param parent
	 */
	private void createBuildForgePortWidgets(Composite parent) {
	    String propValue = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PORT, ""); //$NON-NLS-1$
	    
		fBuildForgePort = createLabeledText(parent, BuildForgeMessages.BuildForgeConfigurationEditor_PORT_LABEL, null, propValue);
		fBuildForgePort.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_PORT_DESC);

		fBuildForgePort.addModifyListener(getBuildForgePortModifiedListener());
	}

	/**
	 * This creates a listener to alert if the port has been modified.
	 * 
	 * @return a ModifyListener
	 */
	private ModifyListener getBuildForgePortModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
			    setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PORT, fBuildForgePort.getText().trim());
			    clearPassword();
				setDirty(true);
			    validate();
			}
		};
	}

	private void clearPassword() {
		if (fBuildForgePassword.getText().length() > 0) {
			fBuildForgePassword.setText(""); //$NON-NLS-1$
		}
	    setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PASSWORD, ""); //$NON-NLS-1$ // paranoia
	}

	/**
	 * This creates the widgets for the user ID in the Build Forge tab.
	 * 
	 * @param parent
	 */
	private void createBuildForgeUserIdWidgets(Composite parent) {
	    String propValue = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_USERID, ""); //$NON-NLS-1$

		fBuildForgeUserId = createLabeledText(parent, BuildForgeMessages.BuildForgeConfigurationEditor_USER_ID_LABEL, null, propValue);
		fBuildForgeUserId.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_USER_ID_DESC);
		
		fBuildForgeUserId.addModifyListener(getBuildForgeUserIdModifiedListener());
	}

	/**
	 * This creates a listener to alert if the user ID has been modified.
	 * 
	 * @return a ModifyListener
	 */
	private ModifyListener getBuildForgeUserIdModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
			    setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_USERID, fBuildForgeUserId.getText().trim());
				setDirty(true);
			    validate();
			}
		};
	}

	/**
	 * This creates the password widgets for the Build Forge tab.
	 * 
	 * @param parent
	 */
	private void createBuildForgePasswordWidgets(Composite parent) {
	    String propValue = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PASSWORD, ""); //$NON-NLS-1$

		// Label
		fToolkit.createLabel(parent, BuildForgeMessages.BuildForgeConfigurationEditor_PASSWORD_LABEL);

		// Text field
		fBuildForgePassword = fToolkit.createText(parent, "", SWT.PASSWORD); //$NON-NLS-1$
		fBuildForgePassword.setLayoutData(new TableWrapData(TableWrapData.FILL));
		fBuildForgePassword.setToolTipText(BuildForgeMessages.BuildForgeConfigurationEditor_PASSWORD_DESC);

		String passwordString = ""; //$NON-NLS-1$
        try {
            passwordString = ObfuscationHelper.decryptString(propValue);
            fBuildForgePassword.setText(passwordString);
        } catch (GeneralSecurityException gse) {
            passwordString = propValue;
            addErrorMessage(fBuildForgePassword, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_DECODING_PASSWORD, fBuildForgePassword);
        } catch (Throwable t) {
            addErrorMessage(fBuildForgePassword, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_DECODING_PASSWORD, fBuildForgePassword);
        }

		fBuildForgePassword.addModifyListener(getBuildForgePasswordModifiedListener());

	}

	/**
	 * Creates a listener to alert for changes in the passwords.
	 * 
	 * @return a ModifyListener
	 */
	private ModifyListener getBuildForgePasswordModifiedListener() {
		return new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				String passwordString = ""; //$NON-NLS-1$
				try {
					passwordString = ObfuscationHelper.encryptString(fBuildForgePassword.getText().trim());
				} catch (GeneralSecurityException gse) {
					passwordString = fBuildForgePassword.getText().trim();
					addErrorMessage(fBuildForgePassword, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_DECODING_PASSWORD, fBuildForgePassword);
				} catch (Throwable t) {
					addErrorMessage(fBuildForgePassword, BuildForgeMessages.BuildForgeConfigurationEditor_ERROR_DECODING_PASSWORD, fBuildForgePassword);
				}
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PASSWORD, passwordString);
				setDirty(true);
				validate();
			}
		};
	}

	/**
	 * Creates the secure connection check box widget.
	 * 
	 * @param parent
	 */
	private void createBuildForgeSecureConnectWidgets(Composite parent) {
	    String propValue = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_SECURE, String.valueOf(true));

		fBuildForgeConnectSecure = fToolkit.createButton(parent, BuildForgeMessages.BuildForgeConfigurationEditor_SECURE_CONNECT_BUTTON, SWT.CHECK);
		TableWrapData layoutData = new TableWrapData(TableWrapData.LEFT);
		layoutData.colspan = 2;
		fBuildForgeConnectSecure.setLayoutData(new TableWrapData(TableWrapData.FILL, TableWrapData.MIDDLE, 1, 2));
		fBuildForgeConnectSecure.setSelection(Boolean.valueOf(propValue));
		fBuildForgeConnectSecure.addSelectionListener(getBuildForgeSecureConnectSelectionListener());
		
        Label descriptionLabel = fToolkit.createLabel(parent, BuildForgeMessages.BuildForgeConfigurationEditor_SECURE_CONNECT_BUTTON_DESC, SWT.WRAP);
        layoutData = new TableWrapData(TableWrapData.FILL, TableWrapData.FILL);
        layoutData.colspan = 2;
        layoutData.maxWidth = 250; // magic!
        descriptionLabel.setLayoutData(layoutData);
        descriptionLabel.setForeground(Display.getDefault().getSystemColor(SWT.COLOR_DARK_GRAY));
	}

	/**
	 * Creates a listener to alert when the secure connection widget is
	 * selected.
	 * 
	 * @return a SelectionListener
	 */
	protected SelectionListener getBuildForgeSecureConnectSelectionListener() {
		return new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
			    boolean secure = fBuildForgeConnectSecure.getSelection();
				setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_SECURE, Boolean.toString(secure));
			    String portProp = getConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PORT, Integer.toString(BuildForgeConstants.DEFAULT_SERVICES_LAYER_TCP_PORT));
				int portInt = Integer.parseInt(portProp);
				// Nick changed: now sets port config property in addition to text field
				if (secure && portInt == BuildForgeConstants.DEFAULT_SERVICES_LAYER_TCP_PORT) {
					String securePort = String.valueOf(BuildForgeConstants.DEFAULT_SERVICES_LAYER_SSL_PORT);
					fBuildForgePort.setText(securePort);
					setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PORT, securePort);
				} else if (!secure && portInt == BuildForgeConstants.DEFAULT_SERVICES_LAYER_SSL_PORT) {
					String port = String.valueOf(BuildForgeConstants.DEFAULT_SERVICES_LAYER_TCP_PORT);
					fBuildForgePort.setText(port);
                    setConfigProperty(BuildForgeConfigurationElement.PROPERTY_BUILDFORGE_PORT, port);
				}
				setDirty(true);
			}
		};
	}

	@Override
	public boolean validate() {

		boolean validHostname = validateHostname();
		boolean validPort = validatePort();
		boolean validUser = validateUserId();
		boolean validPassword = validatePassword();
		
		boolean isValid = validHostname && validPort && validUser && validPassword;
		setPageStatusIndicator(!isValid, false);
		return isValid;
	}

	/**
	 * Validates that the hostname field has a value.
	 * 
	 * @return if the hostname validates successfully
	 */
	private boolean validateHostname() {
		boolean isValid = true;

		// TODO: Need to add some validation logic for IPv4, IPv6, or hostname
		// formats
		if (fBuildForgeHost.getText().trim().isEmpty()) {
			addErrorMessageForRequiredField(fBuildForgeHost, BuildForgeMessages.BuildForgeConfigurationEditor_HOST_NAME_REQUIRED, fBuildForgeHost);
			isValid = false;
		} else {
			removeMessage(fBuildForgeHost, fBuildForgeHost);
		}

		return isValid;
	}

	/**
	 * Validates that the port field has a value, that the value is numeric, and
	 * that the value is in the range of 0 to 65535.
	 * 
	 * @return if the port validates successfully
	 */
	private boolean validatePort() {
		boolean isValid = true;
		String portString = fBuildForgePort.getText().trim();

		if (portString.isEmpty()) {
			addErrorMessageForRequiredField(fBuildForgePort, BuildForgeMessages.BuildForgeConfigurationEditor_PORT_REQUIRED, fBuildForgePort);
			isValid = false;
		} else {
			removeMessage(fBuildForgePort, fBuildForgePort);
			if (!portString.matches("^\\d+$")) { //$NON-NLS-1$
				addErrorMessage(fBuildForgePort, BuildForgeMessages.BuildForgeConfigurationEditor_PORT_MUST_BE_NUMBER, fBuildForgePort);
				isValid = false;
			} else {
				try {
					Integer portInt = new Integer(portString);
					if (portInt < 1 || portInt > 65535) {
						addErrorMessage(fBuildForgePort, BuildForgeMessages.BuildForgeConfigurationEditor_PORT_MUST_BE_NUMBER, fBuildForgePort);
						isValid = false;
					}
				} catch (NumberFormatException nfe) {
					addErrorMessage(fBuildForgePort, BuildForgeMessages.BuildForgeConfigurationEditor_PORT_MUST_BE_NUMBER, fBuildForgePort);
					isValid = false;
				}
			}
		}

		return isValid;
	}

	/**
	 * Validates that the user ID field has a value.
	 * 
	 * @return if the userid validates successfully.
	 */
	private boolean validateUserId() {
		boolean isValid = true;

		if (fBuildForgeUserId.getText().trim().isEmpty()) {
			addErrorMessageForRequiredField(fBuildForgeUserId, BuildForgeMessages.BuildForgeConfigurationEditor_USER_ID_REQUIRED, fBuildForgeUserId);
			isValid = false;
		} else {
			removeMessage(fBuildForgeUserId, fBuildForgeUserId);
		}

		return isValid;
	}

	/**
	 * Validates that the password field has a value.
	 * 
	 * @return if the password validates successfully.
	 */
	private boolean validatePassword() {
		boolean isValid = true;

		if (fBuildForgePassword.getText().trim().isEmpty()) {
			addErrorMessageForRequiredField(fBuildForgePassword, BuildForgeMessages.BuildForgeConfigurationEditor_PASSWORD_REQUIRED, fBuildForgePassword);
			isValid = false;
		} else {
			removeMessage(fBuildForgePassword, fBuildForgePassword);
		}

		return isValid;
	}

	@Override
	public Control getFocusControl() {
		return fBuildForgeConnectSecure;
	}

	@Override
	protected String getContextHelpId() {
		return IHelpContextIds.HELP_CONTEXT_BUILD_ENGINE_EDITOR_BUILD_FORGE_PAGE;
	}

}
