/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.rational.connector.hudson.internal.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.rational.connector.hudson.internal.common.IHudsonService;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Implementation of the Hudson service client. This will be used by the client
 * to communicate the server to get the Hudson information.
 */
public class HudsonServiceClient implements IHudsonServiceClient {

	private IClientLibraryContext fContext;

	/**
	 * Instantiates the Hudson client.
	 * 
	 * @param context
	 *            - Library context to set
	 */
	public HudsonServiceClient(final IClientLibraryContext context) {
		fContext = context;
	}

	/**
	 * Gets the Hudson service from the server.
	 * 
	 * @return IHudsonService - get this Service object
	 * @throws TeamRepositoryException
	 */
	private IHudsonService getService() throws TeamRepositoryException {
		IHudsonService service = (IHudsonService) fContext.getServiceInterface(IHudsonService.class);
		if (service == null) {
			throw new TeamRepositoryException(Messages.HudsonServiceClient_SERVICE_NOT_FOUND);
		}
		return service;
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public IBuildProperty[] requestConnectionTest(final IBuildEngine connectionDetails,
			final IProgressMonitor progressMonitor) throws IllegalArgumentException, TeamRepositoryException {
		IServiceRunnable<Object> runnable = new IServiceRunnable<Object>() {
			public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
				return getService().requestConnectionTest2(connectionDetails);
			}
		};

		return (IBuildProperty[]) fContext.callCancelableService(runnable, progressMonitor);
	}

	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public IBuildDefinition[] getJobs(final IBuildEngine[] buildEngines, final IProgressMonitor progressMonitor)
			throws TeamRepositoryException {
		IServiceRunnable<Object> runnable = new IServiceRunnable<Object>() {
			public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
				return (IBuildDefinition[]) getService().getJobs(buildEngines);
			}
		};

		return (IBuildDefinition[]) fContext.callCancelableService(runnable, progressMonitor);
	}
	
	/*
	 * (non-Javadoc) Intentionally not documented. See parent.
	 */
	public boolean parameterizeHudsonJob(
			final IBuildEngine buildEngine,
			final String jobName,
			final IProgressMonitor progressMonitor) throws TeamRepositoryException {
		IServiceRunnable<Object> runnable = new IServiceRunnable<Object>() {
			public Object run(IProgressMonitor monitor)
					throws TeamRepositoryException {
				return new Boolean(getService().parameterizeHudsonJob3(
						buildEngine, jobName));
			}
		};
		return (Boolean) fContext.callCancelableService(runnable,
				progressMonitor);
	}

}
