/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.rational.connector.hudson.internal.client;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Client side services object.
 */
public interface IHudsonServiceClient {

	/**
	 * Requests a connection test via the server side Hudson service object.
	 * Returns the results of the connection test as an array of
	 * build properties.
	 * 
	 * @param connectionDetails
	 *            A build engine with configuration properties capturing the
	 *            connection details
	 * @param progressMonitor
	 *            UI progress monitor to report the progress to
	 * @return IBuildProperty[] - Array of data returned from the test
	 *         connection
	 * @throws TeamRepositoryException
	 * @throws IllegalArgumentException
	 */
	public IBuildProperty[] requestConnectionTest(
			IBuildEngine connectionDetails,
			IProgressMonitor progressMonitor) throws TeamRepositoryException,
			IllegalArgumentException;

	/**
	 * Gets the Hudson jobs via the server side Hudson service object. This
	 * method takes the first engine and gathers the list of Jobs on that Hudson
	 * server. The result is returned as an array of IBuildDefinitions.
	 * 
	 * @param buildEngines
	 *            Array of engines to look for Hudson Jobs on.
	 * @param progressMonitor
	 *            UI progess monitor to report the progress to
	 * @return IBuildDefinition[] - Array of Hudson Jobs available from the
	 *         given engines.
	 * @throws TeamRepositoryException
	 */
	public IBuildDefinition[] getJobs(final IBuildEngine[] buildEngines,
			IProgressMonitor progressMonitor) throws TeamRepositoryException;
	
	/**
	 * This method will make the Hudson job parameterized if it is not already, and will add in the RTC specific parameters.
	 * 
	 * @param buildEngine The build engine that has access information about the Hudson/Jenkins system
	 * which has the Job defined.
	 * @param jobName The name of the Hudson/Jenkins job that is to be parameterized.
	 * @return Whether the job was updated to contain the parameter. <code>true</code>
	 * the parameter(s) were added <code>false</code> the parameter(s) were already there.
	 * @throws TeamRepositoryException 
	 */
	public boolean parameterizeHudsonJob(IBuildEngine buildEngine, String jobName, IProgressMonitor progressMonitor) throws TeamRepositoryException;

}
