/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui;

import org.eclipse.core.resources.IFile;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.ui.IEditorSite;

import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.hyperlink.AbstractWorkspaceFileHyperlink;

/**
 * Abstract base class for hyperlinks representing java source files in the
 * workspace.
 */
public abstract class AbstractJavaSourceHyperlink extends AbstractWorkspaceFileHyperlink {

    /**
     * Create a new AbstractJavaSourceHyperlink
     * 
     * @param editorSite
     *            the {@link IEditorSite} in which the hyperlink exists.
     */
    public AbstractJavaSourceHyperlink(IEditorSite editorSite) {
        super(editorSite);
    }

    /**
     * Retrieve the {@link IFile} representing the workspace file to open.
     * 
     * @return the {@link IFile} representing the workspace file to open.
     */
    public IFile getWorkspaceFile() {
        IFile file = null;

        try {
            file = JdtWorkspaceHelper.findJavaFile(getTypeName());
        } catch (JavaModelException exception) {
            BuildUIPlugin.log(exception);
        }

        return file;
    }

    /**
     * Get the fully qualified type name of the java class to open
     * 
     * @return the fully qualified type name of the class to open
     */
    public abstract String getTypeName();
}
