/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.ICompileSource;
import com.ibm.team.build.internal.java.ui.editors.result.compile.CompileSourceNode;

/**
 * Open the java sourcefile specified in the {@link CompileSourceNode}
 */
public class OpenCompileSourceActionDelegate extends ActionDelegate implements IObjectActionDelegate {

    private IStructuredSelection fSelection;
    private IWorkbenchPart fTargetPart;

	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			fSelection = (IStructuredSelection) selection;
			// We only allow single selection
			if (action != null) {
				action.setEnabled(fSelection.size() == 1);
			}
		}
	}

    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fTargetPart = targetPart;
    }

    public void run(IAction action) {
        if (fSelection.getFirstElement() instanceof CompileSourceNode) {
            ICompileSource compileSource = ((CompileSourceNode) fSelection.getFirstElement()).getCompileSource();
            openCompileSource(compileSource, 1, fTargetPart.getSite().getPage());
        }
    }

    protected IStructuredSelection getSelection() {
        return fSelection;
    }

    protected IWorkbenchPart getActivePart() {
        return fTargetPart;
    }

    /**
     * Open the java file indicated by the {@link ICompileSource} object.
     * 
     * @param compileSource
     *            the compile source to open
     * @param lineNumber
     *            the line in the source file to navigate to
     * @param workbenchPage
     *            the workbench page
     */
    protected void openCompileSource(ICompileSource compileSource, int lineNumber, IWorkbenchPage workbenchPage) {
    	openSourceFile(compileSource, lineNumber, workbenchPage);
    }

    protected void openSourceFile(ICompileSource compileSource, int lineNumber, IWorkbenchPage workbenchPage) {
        JdtWorkspaceHelper.openSourceFile(compileSource, lineNumber, workbenchPage.getActiveEditor().getEditorSite());
    }
}
