/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.compile;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Implemented by any compile tree nodes that can provide simple compile details
 * that should show up in the compile details block of the UI.
 */
public interface ICompileDetailProvider {

    /**
     * Return a string containing the error details for this detail provider.
     * 
     * @param monitor
     *            used to report progress and check for cancellation
     * @return A string containing the compile details.
     * @throws TeamRepositoryException
     */
    public String getErrorDetails(IProgressMonitor monitor) throws TeamRepositoryException;

    /**
     * Return a string containing the warning details for this detail provider.
     * 
     * @param monitor
     *            used to report progress and check for cancellation
     * @return A string containing the compile details.
     * @throws TeamRepositoryException
     */
    public String getWarningDetails(IProgressMonitor monitor) throws TeamRepositoryException;

    /**
     * Return true if the detail provider has warning details.
     * 
     * @return True if the provider has warning details.
     */
    public boolean hasWarnings();

    /**
     * Return true if the detail provider has errors details.
     * 
     * @return True if the provider has error details.
     */
    public boolean hasErrors();
}
