/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.compile;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.build.common.model.ICompileProblem;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Implemented by any compile tree nodes that can provide compile details that
 * should show up in the compile details block of the UI.
 */
public interface IComplexCompileDetailProvider extends ICompileDetailProvider {

    /**
     * Return the {@link ICompileProblem} objects representing the errors for
     * this detail provider.
     * 
     * @param monitor
     *            used to report progress and check for cancellation
     * @return An array of {@link ICompileProblem} objects representing the
     *         errors for this detail provider.
     * @throws TeamRepositoryException
     */
    public ICompileProblem[] getErrors(IProgressMonitor monitor) throws TeamRepositoryException;

    /**
     * Return the {@link ICompileProblem} objects representing the warnings for
     * this detail provider.
     * 
     * @param monitor
     *            used to report progress and check for cancellation
     * @return An array of {@link ICompileProblem} objects representing the
     *         warnings for this detail provider.
     * @throws TeamRepositoryException
     */
    public ICompileProblem[] getWarnings(IProgressMonitor monitor) throws TeamRepositoryException;

}