/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.compile;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResultContribution;
import com.ibm.team.build.common.model.ICompileContribution;
import com.ibm.team.build.internal.client.util.ContentUtil;
import com.ibm.team.build.internal.common.helper.ValidationHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * A node representing a simple (non-xml-log) compile contribution. The node
 * contains both the <code>IBuildResultContribution</code> and the
 * <code>ICompileContribution</code>.
 */
public class SimpleCompileContributionNode extends AbstractCompileTreeNode implements ICompileDetailProvider {

    private ICompileContribution fCompileContribution;
    private IBuildResultContribution fBuildResultContribution;

    /**
     * Create a simple compile contribution node.
     * 
     * @param parentNode
     *            the parent node of this node
     * @param buildResultContribution
     *            the build result contribution
     * @param compileContribution
     *            the compile contribution
     */
    public SimpleCompileContributionNode(ICompileTreeNode parentNode, IBuildResultContribution buildResultContribution,
            ICompileContribution compileContribution) {
        super(parentNode);
        ValidationHelper.validateNotNull("parentNode", parentNode); //$NON-NLS-1$
        ValidationHelper.validateNotNull("buildResultContribution", buildResultContribution); //$NON-NLS-1$
        ValidationHelper.validateNotNull("compileContribution", compileContribution); //$NON-NLS-1$
        fBuildResultContribution = buildResultContribution;
        fCompileContribution = compileContribution;
    }

    @Override
    public boolean hasChildren() {
        return false;
    }

    @Override
    protected Object[] doFetchChildren(IProgressMonitor monitor) throws TeamRepositoryException {
        return null;
    }

    @Override
    public Image getColumnImage(int columnIndex) {
        if (columnIndex == CompileResultsPage.NAME_COLUMN_INDEX) {
            Image newImage = BuildUIPlugin.getImage("icons/view16/compile.gif"); //$NON-NLS-1$

            BuildStatus status = BuildStatus.OK;
            if (fCompileContribution.getWarningCount() > 0) {
                status = BuildStatus.WARNING;
            }
            if (fCompileContribution.getErrorCount() > 0) {
                status = BuildStatus.ERROR;
            }

            return BuildCompositeImageRegistry.getImage(newImage, status, false);
        }

        return null;
    }

    @Override
    public String getColumnText(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.NAME_COLUMN_INDEX:
                return getNodeName();
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return Integer.toString(fCompileContribution.getWarningCount());
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return Integer.toString(fCompileContribution.getErrorCount());
        }
        return null;
    }

    @Override
    public Long getColumnAsLong(int columnIndex) {
        switch (columnIndex) {
            case CompileResultsPage.WARNING_COUNT_COLUMN_INDEX:
                return (long) fCompileContribution.getWarningCount();
            case CompileResultsPage.ERROR_COUNT_COLUMN_INDEX:
                return (long) fCompileContribution.getErrorCount();
        }
        return null;
    }
    
    @Override
    public boolean hasProblems() {
        return fCompileContribution.getErrorCount() > 0 || fCompileContribution.getWarningCount() > 0;
    }

    @Override
    public String getNodeName() {
        return fBuildResultContribution.getLabel();
    }

    @Override
    public boolean hasWarnings() {
        return fCompileContribution.getWarningCount() > 0;
    }

    @Override
    public boolean hasErrors() {
        return fCompileContribution.getErrorCount() > 0;
    }

    @Override
    public String getErrorDetails(IProgressMonitor monitor) throws TeamRepositoryException {
        if (fCompileContribution.getErrors() != null) {
            return ContentUtil.contentToString((ITeamRepository) fCompileContribution.getOrigin(),
                    fCompileContribution.getErrors(), monitor);
        }
        return null;
    }

    @Override
    public String getWarningDetails(IProgressMonitor monitor) throws TeamRepositoryException {
        if (fCompileContribution.getWarnings() != null) {
            return ContentUtil.contentToString((ITeamRepository) fCompileContribution.getOrigin(),
                    fCompileContribution.getWarnings(), monitor);
        }
        return null;
    }
}
