/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.junit;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.texteditor.IUpdate;

import com.ibm.team.build.internal.ui.helper.ClipboardHelper;

/**
 * Command to copy the text in the viewer to the clipboard.
 */
public class CopyToClipboardAction extends Action implements IUpdate {

    private SourceViewer fViewer;

    /**
     * Create a new action that will copy text from viewer
     * 
     * @param viewer
     *            {@link SourceViewer} to copy text from.
     */
    public CopyToClipboardAction(SourceViewer viewer) {
        fViewer = viewer;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.eclipse.jface.action.IAction#run()
     */
    public void run() {
        BusyIndicator.showWhile(Display.getCurrent(), new Runnable() {
            public void run() {
                ClipboardHelper.getDefault().copyTextToClipboard(getStackTrace(), getViewer().getControl().getDisplay());
            }
        });
    }

    private String getStackTrace() {
        return getViewer().getDocument().get();
    }

    protected SourceViewer getViewer() {
        return fViewer;
    }

    public void update() {
        setEnabled(getStackTrace().length() != 0);
    }

}
