/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.java.ui.editors.result.junit;

import com.ibm.team.repository.common.IContent;

/**
 * Extends {@link IJUnitDetailProvider} to add information about errors that
 * occurred in a JUnit run.
 */
public interface IJUnitRunErrorDetailProvider extends IJUnitDetailProvider {

    /**
     * Return whether or not an error occurred during this JUnit test run.
     * 
     * @return boolean
     */
    public boolean isError();

    /**
     * Return the actual error data if an error occurred during this JUnit test
     * run.
     * 
     * @return {@link IContent}
     */
    public IContent getErrorData();

    /**
     * Return whether or not a test failure occurred during this JUnit test run.
     * 
     * @return boolean
     */
    public boolean isFailure();

    /**
     * Return the actual failure data if an failure occurred during this JUnit
     * test run.
     * 
     * @return {@link IContent}
     */
    public IContent getFailureData();

    /**
     * Return the detail string if it has been set.
     * 
     * @return the detail string if it has been set.
     */
    public String getDetailString();

    /**
     * Set the detail string from the fetched content so the detail provider can
     * cache it.
     * 
     * @param detailString
     *            The detail provider can cache this and return it from
     *            getDetailString(String)
     */
    public void setDetailString(String detailString);

}
