/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.layout.TreeColumnLayout;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.forms.editor.FormEditor;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IJUnitTestSuiteContribution;
import com.ibm.team.build.internal.java.ui.JdtWorkspaceHelper;
import com.ibm.team.build.internal.ui.BuildCompositeImageRegistry;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.IHelpContextIds;
import com.ibm.team.build.internal.ui.editors.result.BuildLabelProvider;
import com.ibm.team.build.internal.ui.editors.result.BuildResultEditorMessages;
import com.ibm.team.build.internal.ui.editors.result.BuildResultPage;
import com.ibm.team.build.internal.ui.editors.result.BuildResultsTreeViewerComparator;
import com.ibm.team.build.internal.ui.editors.result.IDeferredBuildTreeNode;
import com.ibm.team.build.ui.editors.result.IBuildResultContext;

/**
 * A build result tab for JUnit results.
 */
public class JUnitResultsPage extends BuildResultPage {

    protected IAction fOpenAction;

    /**
     * Context menu ID for the junit results page.
     */
    public final static String CONTEXT_MENU_ID = "com.ibm.team.build.ui.junitPageContextMenu"; //$NON-NLS-1$

    protected boolean fShowSuccessNodes;

    /**
     * Create the JUnit results tab.
     * 
     * @param editor
     *            the form editor
     * @param id
     *            the page id
     * @param title
     *            the page title
     * @param buildResultContext
     *            the context that provides access to data related to the build
     *            result
     */
    public JUnitResultsPage(FormEditor editor, String id, String title, IBuildResultContext buildResultContext) {
        super(editor, id, title, new String[] { IJUnitTestSuiteContribution.EXTENDED_CONTRIBUTION_TYPE_ID },
                buildResultContext);

        fOpenAction = new Action(BuildResultEditorMessages.JUnitResultsPage_OPEN_ACTION_NAME) {
            public void run() {
                openSelection(getSelectedItem());
            }
        };

        fShowSuccessNodes = !containsFailures(getContributions());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public PageGroup getPageGroup() {
        return PageGroup.TESTS;
    }

    @Override
    protected boolean addActionsToContextMenu(IMenuManager manager) {
        if (getSelectionCount() == 1 && getSelectedItem() instanceof IJUnitDetailProvider) {
            manager.add(fOpenAction);
            return true;
        }

        return false;
    }

    /**
     * Class that listens to selection in tree and updates detail section.
     */
    private final class TreeViewerSelectionChangedListener implements ISelectionChangedListener {
        public void selectionChanged(SelectionChangedEvent event) {
            IStructuredSelection selection = (IStructuredSelection) event.getSelection();
            if (selection.size() == 1) {
                Object element = selection.getFirstElement();
                /*
                 * The refresh after a removeFilter call generates a bogus
                 * SelectionEvent. TODO: remove after Eclipse Bug 176106 is
                 * fixed.
                 */
                if (inRemoveFilterCall()) {
                    return;
                }

                if (element instanceof IJUnitDetailProvider) {
                    fDetailBlock.setDetailProvider((IJUnitDetailProvider) element,
                            getBuildResultContext().getTeamRepository());
                    return;
                }
            }
            fDetailBlock.setDetailProvider(null, null);
        }
    }

    protected void openSelection(Object selectedItem) {
        IJUnitDetailProvider provider = (IJUnitDetailProvider) selectedItem;
        String locInfo = null;
        if (provider instanceof JUnitTestCaseNode && ((JUnitTestCaseNode)provider).getDetailString() != null) {
        	locInfo = UntiTestLocationUtils.getLocationInfo(((JUnitTestCaseNode)provider).getDetailString());
        }
        if (provider.getJUnitName().length() > 0) {
        	if(locInfo != null) {
        		openJUnitSourceFile(locInfo, provider.getJUnitClassName(), provider.getJUnitName(), getEditorSite());
        	}else {
        		openJUnitSourceFile(provider.getJUnitClassName(), provider.getJUnitName(), getEditorSite());
        	}
        } else {
        	if(locInfo != null) {
        		openJavaSourceFile(locInfo, provider.getJUnitClassName(), getEditorSite());
        	}else {
        		openJavaSourceFile(provider.getJUnitClassName(), getEditorSite());
        	}
        }
    }

    protected void openJUnitSourceFile(String className, String testName, IEditorSite editorSite) {
        JdtWorkspaceHelper.openJUnitSourceFile(className, testName, editorSite);
    }

    protected void openJavaSourceFile(String className, IEditorSite editorSite) {
        JdtWorkspaceHelper.openJavaSourceFile(className, 1, editorSite);
    }
    
    protected boolean openJUnitSourceFile(String locationInfo, String className,
			String junitTestName, IEditorSite editorSite) {
		return JdtWorkspaceHelper.openJUnitSourceFile(UntiTestLocationUtils.getFilePath(locationInfo), 
				UntiTestLocationUtils.getLineLocation(locationInfo), className, junitTestName,
				editorSite);
	}

	protected boolean openJavaSourceFile(String locationInfo,
			String className, IEditorSite editorSite) {
		return JdtWorkspaceHelper.openJavaSourceFile(
				UntiTestLocationUtils.getFilePath(locationInfo), className, 1, editorSite);
	}

    /**
     * Listener that expands nodes in Junit results tree when they are double
     * clicked.
     */
    private final class TreeViewerOpenListener implements IOpenListener {
        public void open(OpenEvent event) {
            Object element = getSelectedItem();
            if (element instanceof JUnitTestCaseNode) {
                openSelection(element);
            }
        }
    }

    /**
     * Class that listens to selection of treeColumn and sets sort column in
     * tree accordingly.
     */
    private final class TreeColumnSelectionListener implements SelectionListener {

        private final TreeColumn fTreeColumn;
        private final Tree fTree;

        /**
         * Create a listener on the given column and tree
         * 
         * @param treeColumn
         *            {@link TreeColumn} that this listener is listening on.
         * @param tree
         *            {@link Tree} that contains treeColumn.
         */
        private TreeColumnSelectionListener(TreeColumn treeColumn, Tree tree) {
            fTreeColumn = treeColumn;
            fTree = tree;
        }

        public void widgetSelected(SelectionEvent e) {

            if (fTree.getSortColumn() == fTreeColumn) {
                if (fTree.getSortDirection() == SWT.DOWN) {
                    fTree.setSortDirection(SWT.UP);
                } else {
                    fTree.setSortDirection(SWT.DOWN);
                }
            } else {
                fTree.setSortColumn(fTreeColumn);
                fTree.setSortDirection(SWT.DOWN);
            }

            Object[] expandedElements = fViewer.getExpandedElements();
            fViewer.refresh(false);
            fViewer.setExpandedElements(expandedElements);
        }

        public void widgetDefaultSelected(SelectionEvent e) {
            // do nothing
        }
    }

    /**
     * The index of the test count column.
     */
    public final static int TEST_COUNT_COLUMN_INDEX = 1;

    /**
     * The index of the failure count column.
     */
    public final static int FAILURE_COUNT_COLUMN_INDEX = 2;

    /**
     * The index of the error count column.
     */
    public final static int ERROR_COUNT_COLUMN_INDEX = 3;

    /**
     * The index of the time taken column.
     */
    public final static int TIME_TAKEN_COLUMN_INDEX = 4;

    /**
     * The index of the run order column.
     */
    public final static int RUN_ORDER_COLUMN_INDEX = 5;

    private JUnitDetailBlock fDetailBlock;

    private TreeViewer fViewer;

    private Tree fJUnitResultsTree;

    protected ViewerFilter fSuccessNodeFilter = new ViewerFilter() {
        public boolean select(Viewer viewer, Object parentElement, Object element) {
            if (element instanceof IDeferredBuildTreeNode) {
                return ((IDeferredBuildTreeNode) element).hasProblems();
            }
            return true;
        }
    };

    protected void doCreatePageContent(Composite parent) {
        FormToolkit toolkit = getManagedForm().getToolkit();
        toolkit.setBorderStyle(SWT.BORDER);

        FormLayout formLayout = new FormLayout();
        parent.setLayout(formLayout);

        addResultTree(parent, toolkit);
        addDetails(parent, toolkit);
        createContextMenu(CONTEXT_MENU_ID, getSite());

        PlatformUI.getWorkbench().getHelpSystem().setHelp(parent,
                IHelpContextIds.HELP_CONTEXT_BUILD_RESULT_EDITOR_JUNIT_PAGE);
    }

    private void addResultTree(final Composite body, FormToolkit toolkit) {
        Section section = toolkit.createSection(body, Section.TITLE_BAR);
        section.setText(BuildResultEditorMessages.JUnitResultsPage_JUnitTests);
        GridLayoutFactory.fillDefaults().margins(0, 0).applyTo(section);

        ToolBar toolbar = createToolbar(toolkit, section);
        section.setTextClient(toolbar);

        Composite treeComposite = toolkit.createComposite(body);

        FormData formData = new FormData();
        formData.left = new FormAttachment(0, 0);
        formData.top = new FormAttachment(0, 0);
        formData.right = new FormAttachment(100, 0);
        formData.bottom = new FormAttachment(treeComposite);
        section.setLayoutData(formData);

        formData = new FormData();
        formData.left = new FormAttachment(0, 0);
        formData.top = new FormAttachment(section, 7);
        formData.right = new FormAttachment(100, 0);
        formData.bottom = new FormAttachment(50, 0);
        treeComposite.setLayoutData(formData);

        fJUnitResultsTree = toolkit.createTree(treeComposite, SWT.BORDER | SWT.VIRTUAL | SWT.FULL_SELECTION
                | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI);
        fJUnitResultsTree.setLinesVisible(true);
        fJUnitResultsTree.setHeaderVisible(true);
        fJUnitResultsTree.addListener(SWT.SetData, new Listener() {
            public void handleEvent(Event event) {
                fJUnitResultsTree.removeListener(SWT.SetData, this);
                body.layout(true, true);
            }
        });

        TreeColumnLayout treeColumnLayout = new TreeColumnLayout();

        createTreeColumn(fJUnitResultsTree, treeColumnLayout, BuildResultEditorMessages.NAME_COLUMN_TITLE, 50,
                SWT.LEFT, NAME_COLUMN_INDEX);
        createTreeColumn(fJUnitResultsTree, treeColumnLayout, BuildResultEditorMessages.TEST_COUNT_COLUMN_TITLE, 10,
                SWT.RIGHT, TEST_COUNT_COLUMN_INDEX);
        createTreeColumn(fJUnitResultsTree, treeColumnLayout, BuildResultEditorMessages.FAILURE_COUNT_COLUMN_TITLE, 10,
                SWT.RIGHT, FAILURE_COUNT_COLUMN_INDEX);
        createTreeColumn(fJUnitResultsTree, treeColumnLayout, BuildResultEditorMessages.ERROR_COUNT_COLUMN_TITLE, 10,
                SWT.RIGHT, ERROR_COUNT_COLUMN_INDEX);
        createTreeColumn(fJUnitResultsTree, treeColumnLayout, BuildResultEditorMessages.TIME_TAKEN_COLUMN_TITLE, 10,
                SWT.RIGHT, TIME_TAKEN_COLUMN_INDEX);
        createTreeColumn(fJUnitResultsTree, treeColumnLayout, BuildResultEditorMessages.RUN_ORDER_COLUMN_TITLE, 10,
                SWT.RIGHT, RUN_ORDER_COLUMN_INDEX);

        treeComposite.setLayout(treeColumnLayout);

        fViewer = new TreeViewer(fJUnitResultsTree);
        fViewer.setLabelProvider(new BuildLabelProvider());
        fViewer.setContentProvider(new JUnitContentProvider(fViewer));
        fViewer.setComparator(new BuildResultsTreeViewerComparator());

        TreeColumn runOrderColumn = fViewer.getTree().getColumn(RUN_ORDER_COLUMN_INDEX);
        fViewer.getTree().setSortColumn(runOrderColumn);
        fViewer.getTree().setSortDirection(SWT.UP);

        fViewer.addOpenListener(createTreeViewerOpenListener());

        Image showSuccessImage = BuildCompositeImageRegistry.getImage(BuildUIPlugin.getImage("icons/obj16/test_obj.gif"), //$NON-NLS-1$
                BuildStatus.OK, true);

        createFilterButton(showSuccessImage, BuildResultEditorMessages.JUnitResultsPage_FILTER_TOOLTIP, toolbar,
                fSuccessNodeFilter, fViewer, fShowSuccessNodes);

        // Set the input last to avoid refresh conflicts with the previous call
        // to createFilterButton.
        fViewer.setInput(new JUnitBuildResultNode(getContributions()));
    }

    @Override
    protected void toggleFilter(boolean buttonSelected, StructuredViewer viewer, ViewerFilter filter) {
        fShowSuccessNodes = buttonSelected;
        super.toggleFilter(buttonSelected, viewer, filter);
    }

    /**
     * Creates a new listener for the TreeViewer. This method is protected so
     * that tests can access it, it is not meant to be overridden by subclasses.
     * 
     * @return {@link TreeViewerDoubleClickListener}, never null.
     */
    protected IOpenListener createTreeViewerOpenListener() {
        return new TreeViewerOpenListener();
    }

    private void addDetails(Composite body, FormToolkit toolkit) {
        Section section = toolkit.createSection(body, Section.TITLE_BAR);
        section.setText(BuildResultEditorMessages.JUnitResultsPage_Details);
        GridLayoutFactory.fillDefaults().applyTo(section);

        FormData formData = new FormData();
        formData.left = new FormAttachment(0, 0);
        formData.top = new FormAttachment(50, 7);
        formData.right = new FormAttachment(100, 0);
        formData.bottom = new FormAttachment(100, 0);
        section.setLayoutData(formData);

        fDetailBlock = createDetailBlock();
        section.setClient(fDetailBlock.createContents(section, toolkit, getEditor()));
        fViewer.addSelectionChangedListener(createSelectionChangedListener());
    }

    /**
     * Allows access to the UI section that shows details about JUnit tests.
     * <p>
     * This method is protected for testing; it should not be overridden
     * </p>
     * 
     * @return {@link JUnitDetailBlock}
     */
    protected JUnitDetailBlock createDetailBlock() {
        return new JUnitDetailBlock();
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected Viewer getViewer() {
        return fViewer;
    }

    protected Object getSelectedItem() {
        IStructuredSelection selection = (IStructuredSelection) fViewer.getSelection();
        return selection.getFirstElement();
	}

	protected int getSelectionCount() {
		return ((IStructuredSelection) fViewer.getSelection()).size();
	}

    @Override
    protected CopyStrategy getCopyStrategy(Control control) {
        return CopyStrategy.FIRST;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getCopyActionItemNameForSelectedItem(Control control) {
        Object selectedItem = getSelectedItem();

        if (selectedItem instanceof JUnitComponentNode) {
            return BuildResultEditorMessages.JUnitResultsPage_COMPONENT_NAME;
        } else if (selectedItem instanceof JUnitTestSuiteNode) {
            return BuildResultEditorMessages.JUnitResultsPage_TESTSUITE_NAME;
        } else if (selectedItem instanceof JUnitTestClassNode) {
            return BuildResultEditorMessages.JUnitResultsPage_CLASS_NAME;
        } else { // if (selectedItem instanceof JUnitTestCaseNode)
            return BuildResultEditorMessages.JUnitResultsPage_TESTCASE_NAME;
        }
    }

    protected void createTreeColumn(Tree tree, TreeColumnLayout layout, String columnName, int weight, int style,
            final int index) {
        TreeColumn treeColumn = new TreeColumn(tree, style, index);
        treeColumn.setText(columnName);
        layout.setColumnData(treeColumn, new ColumnWeightData(weight));
        treeColumn.addSelectionListener(getColumnSelectionListener(tree, treeColumn));
    }

    /**
     * Creates a new {@link TreeViewerSelectionChangedListener}
     * 
     * @return {@link ISelectionChangedListener} to be used by a
     *         {@link TreeViewer}, never null.
     */
    protected ISelectionChangedListener createSelectionChangedListener() {
        return new TreeViewerSelectionChangedListener();
    }

    /**
     * Creates a {@link TreeColumnSelectionListener} with the given treeColumn
     * and tree.
     * 
     * @param treeColumn
     *            {@link TreeColumn} this listener is listening for.
     * @param tree
     *            {@link Tree} treeColumn is in.
     * @return new {@link TreeColumnSelectionListener}
     */
    protected SelectionListener createTreeColumnSelectionListener(TreeColumn treeColumn, Tree tree) {
        return new TreeColumnSelectionListener(treeColumn, tree);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getContextHelpId() {
        return IHelpContextIds.HELP_CONTEXT_BUILD_RESULT_EDITOR_JUNIT_PAGE;
    }
}