/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.hyperlink.IHyperlink;
import org.eclipse.jface.text.hyperlink.IHyperlinkDetector;
import org.eclipse.ui.IEditorSite;

import com.ibm.team.build.internal.ui.BuildUIPlugin;

/**
 * Detects hyperlinks in Java stack trace text.
 */
public class JavaStackTraceHyperlinkDetector implements IHyperlinkDetector {

    IEditorSite fEditorSite;

    /**
     * Create a JavaStackTraceHyperlinkDetector object. The specified editor
     * site is passed on to the created hyperlink objects.
     * 
     * @param editorSite
     *            the editor site to pass on to the hyperlinks.
     */
    public JavaStackTraceHyperlinkDetector(IEditorSite editorSite) {
        fEditorSite = editorSite;
    }

    public IHyperlink[] detectHyperlinks(ITextViewer textViewer, final IRegion region, boolean canShowMultipleHyperlinks) {
        if (region == null) {
            return null;
        }
        try {
            IDocument document = textViewer.getDocument();
            IRegion entireLine = document.getLineInformation(document.getLineOfOffset(region.getOffset()));
            String string = document.get(entireLine.getOffset(), entireLine.getLength());
            if (string.matches("\\sat\\s.*\\(.*\\.java\\:\\d*\\)")) { //$NON-NLS-1$
                // Crop the entire line to remove " at " from the start
                IRegion linkRegion = new Region(entireLine.getOffset() + 4, entireLine.getLength() - 4);
                string = string.substring(4);
                return new IHyperlink[] { new JavaStackTraceHyperlinkNoConsole(string, linkRegion, fEditorSite) };
            }
        } catch (BadLocationException exception) {
            BuildUIPlugin.log(exception);
        }
        return null;
    }

}
