/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.editors.result.junit;

import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.text.ITextOperationTarget;
import org.eclipse.ui.texteditor.IUpdate;

/**
 * An action for use in the JUnit page.
 */
public class ViewAction extends Action implements IUpdate {

    /** The text operation code */
    private int fOperationCode = -1;
    /** The text operation target */
    private ITextOperationTarget fOperationTarget;
    /** The text operation target provider */
    private IAdaptable fTargetProvider;

    /**
     * Create the action.
     * 
     * @param target
     *            The target.
     * @param operationCode
     *            The operation code.
     */
    public ViewAction(ITextOperationTarget target, int operationCode) {
        super();
        fOperationTarget = target;
        fOperationCode = operationCode;
        update();
    }

    /**
     * Create the action.
     * 
     * @param targetProvider
     *            The target provider.
     * @param operationCode
     *            The operation code.
     */
    public ViewAction(IAdaptable targetProvider, int operationCode) {
        super();
        fTargetProvider = targetProvider;
        fOperationCode = operationCode;
        update();
    }

    /**
     * The <code>TextOperationAction</code> implementation of this
     * <code>IAction</code> method runs the operation with the current
     * operation code.
     */
    public void run() {
        if (fOperationCode != -1 && fOperationTarget != null)
            fOperationTarget.doOperation(fOperationCode);
    }

    /**
     * The <code>TextOperationAction</code> implementation of this
     * <code>IUpdate</code> method discovers the operation through the current
     * editor's <code>ITextOperationTarget</code> adapter, and sets the
     * enabled state accordingly.
     */
    public void update() {
        if (fOperationTarget == null && fTargetProvider != null && fOperationCode != -1) {
            fOperationTarget = (ITextOperationTarget) fTargetProvider.getAdapter(ITextOperationTarget.class);
        }

        boolean isEnabled = (fOperationTarget != null && fOperationTarget.canDoOperation(fOperationCode));
        setEnabled(isEnabled);
    }
}
