/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.java.ui.workitems;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Display;

import com.ibm.team.build.internal.java.ui.editors.result.compile.ICompileDetailProvider;
import com.ibm.team.build.internal.ui.editors.result.workitem.WorkItemHelper;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.workitem.rcp.ui.WorkItemUI;

/**
 * Action to find similar work items for a compile problem in the build result
 * editor.
 */
public class FindSimilarWorkItemsFromCompileAction extends CreateWorkItemFromCompileAction {

    @Override
    public void run(IAction action) {
        ITeamRepository teamRepository = (ITeamRepository) fBuildResult.getOrigin();
        final Job job = new TeamBuildJob(Messages.FindSimilarWorkItemsFromCompileAction_JOB_LABEL, true, teamRepository) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                ICompileDetailProvider compileDetail = getCompileDetail(fSelection);
                IProjectAreaHandle projectAreaHandle = getProjectAreaHandle();
                if (projectAreaHandle != null) {
                    showRelatedWorkItems(getSummary(compileDetail), getDescription(compileDetail, monitor),
                            projectAreaHandle);
                }
                return Status.OK_STATUS;
            }

            @Override
            protected void jobFinished(IStatus status) {
                handleJobDone(status);
            }
        };

        job.schedule();
    }

    protected void showRelatedWorkItems(String summary, String description, IProjectAreaHandle projectAreaHandle) {
        WorkItemHelper.getInstance().showRelatedWorkItems(summary, description, fBuildResult, projectAreaHandle,
                fSite.getWorkbenchWindow());
    }

    protected IProjectAreaHandle getProjectAreaHandle() {
        final IProjectAreaHandle[] projectArea = new IProjectAreaHandle[1];

        Display.getDefault().syncExec(new Runnable() {
            public void run() {
                projectArea[0] = WorkItemUI.selectProjectArea(fSite.getShell());
            }
        });

        return projectArea[0];
    }
}
