/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;

import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.internal.ui.dialogs.EditBuildDefinitionsDialog;
import com.ibm.team.build.internal.ui.dialogs.EditBuildEnginesDialog;
import com.ibm.team.build.internal.ui.editors.builddefinition.GeneralConfigurationEditor;
import com.ibm.team.build.internal.ui.editors.buildengine.BuildEngineEditor;

/**
 * Helper class to sort the items in various build ui tables:
 * {@link BuildEngineEditor}, {@link GeneralConfigurationEditor},
 * {@link EditBuildDefinitionsDialog}, {@link EditBuildEnginesDialog}
 */
public abstract class AbstractBuildItemViewerComparator extends ViewerComparator {

    /**
     * Constructor for the class
     */
    public AbstractBuildItemViewerComparator() {
        // do nothing
    }

    private int compare(String s1, String s2) {
        return getComparator().compare(s1, s2);
    }
    
    private String getId(Object o) {
        if (o instanceof String) {
            return (String) o;
        } else if (o instanceof IBuildDefinition) {
            return ((IBuildDefinition) o).getId();
        } else {
            return o.toString();
        }
    }

    private int compareEntry(Object o1, Object o2) {
        String projectArea1 = getProject(o1);
        String projectArea2 = getProject(o2);
        if (projectArea1 == null) {
            if (projectArea2 != null) {
                return 1;
            } else {
                return compare(getId(o1), getId(o2));
            }
        } else {
            if (projectArea2 == null) {
                return -1;
            } else {
                int projectAreaCompareValue = compare(projectArea1, projectArea2);
                if (projectAreaCompareValue == 0) {
                    return compare(getId(o1), getId(o2));
                } else {
                    if (projectArea1.equals(getTopProject())) {
                        return -1;
                    } else if (projectArea2.equals(getTopProject())) {
                        return 1;
                    } else {
                        return projectAreaCompareValue;
                    }
                }
            }
        }
    }
    
    @Override
    public int compare(Viewer viewer, Object e1, Object e2) {
        return compareEntry(e1, e2);
    }
    
    protected abstract String getProject(Object element);
    
    protected abstract String getTopProject();
}
