/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;

import com.ibm.team.build.common.model.BuildState;

/**
 * A BuildActivityCompositeImageDescriptor consists of a main icon and an
 * adornment based on the specified status.
 */
public class BuildActivityCompositeImageDescriptor extends AbstractBuildCompositeImageDescriptor {

    /**
     * Create a new BuildActivityCompositeImageDescriptor.
     * 
     * @param baseImage
     *            an image descriptor used as the base image
     * @param state
     *            the build state used to indicate which adornment will be
     *            rendered
     */
    public BuildActivityCompositeImageDescriptor(Image baseImage, BuildState state) {
        super(baseImage, state);
    }

    /**
     * Return the overlay specified by the build state.
     */
    protected ImageData getOverlay() {
        ImageData data = null;
        BuildState state = (BuildState) getAdornmentObject();

        switch (state) {
            case COMPLETED:
                data = BuildUIPlugin.getImageDescriptor("icons/ovr16/bld_act_complt_ovr.gif").getImageData(); //$NON-NLS-1$
                break;
            case IN_PROGRESS:
                data = BuildUIPlugin.getImageDescriptor("icons/ovr16/bld_act_inprgrs_ovr.gif").getImageData(); //$NON-NLS-1$
                break;
            case INCOMPLETE:
                data = BuildUIPlugin.getImageDescriptor("icons/ovr16/bld_act_uncomplt_ovr.gif").getImageData(); //$NON-NLS-1$
        }

        return data;
    }
}
