/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ResourceManager;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;

import com.ibm.team.build.common.model.BuildState;
import com.ibm.team.build.common.model.BuildStatus;
import com.ibm.team.build.common.model.IBuildResult;

/**
 * A BuildStatusCompositeImageDescriptor consists of a main icon and an
 * adornment based on the specified status.
 */
public class BuildStatusCompositeImageDescriptor extends AbstractBuildCompositeImageDescriptor {

    /**
     * Create a new BuildStatusCompositeImageDescriptor.
     * 
     * @param baseImage
     *            an image descriptor used as the base image
     * @param status
     *            the status of the build - indicates which adornments will be
     *            rendered
     */
    public BuildStatusCompositeImageDescriptor(Image baseImage, BuildStatus status) {
        super(baseImage, status);
    }

    /**
     * Add any overlays to the image as specified in the flags.
     */
    protected ImageData getOverlay() {
        ImageData data = null;
        BuildStatus status = (BuildStatus) getAdornmentObject();

        switch (status) {
            case ERROR:
                data = BuildUIPlugin.getImageDescriptor("icons/ovr16/failed_ovr.gif").getImageData(); //$NON-NLS-1$
                break;
            case OK:
                data = BuildUIPlugin.getImageDescriptor("icons/ovr16/success_ovr.gif").getImageData(); //$NON-NLS-1$
                break;
            case WARNING:
                data = BuildUIPlugin.getImageDescriptor("icons/ovr16/warn_ovr.gif").getImageData(); //$NON-NLS-1$
        }

        return data;
    }

    private static ImageDescriptor getBuildResultCompositeImageDescriptor(IBuildResult result) {
        ImageDescriptor desc = null;

        if (result == null) {
            desc = BuildUIPlugin.getImageDescriptor("icons/obj16/abanblds_obj.gif"); //$NON-NLS-1$
        } else if (result.getState() == BuildState.INCOMPLETE) {
            desc = BuildUIPlugin.getImageDescriptor("icons/obj16/abanblds_obj.gif"); //$NON-NLS-1$
        } else {
            Image newImage = BuildUIPlugin.getImage("icons/obj16/bld_rslt.gif"); //$NON-NLS-1$
            desc = new BuildStatusCompositeImageDescriptor(newImage, result.getStatus());
        }

        return desc;
    }

    /**
     * Returns a composite image based on the status of the supplied build
     * result. The given resource manager is used to create the image.
     * 
     * @param result
     *            The build result.
     * @param resourceManager
     *            The resource manager to use for creating the image.
     * @return an image
     */
    public static Image getBuildResultComposite(IBuildResult result, ResourceManager resourceManager) {
        return resourceManager.createImage(getBuildResultCompositeImageDescriptor(result));
    }
}