/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

import org.eclipse.core.runtime.Status;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.repository.common.Location;

/**
 * The localized messages for the package
 * <code>com.ibm.team.build.internal.ui</code>.
 */
public class BuildUIMessages extends NLS {
    private static final String BUNDLE_NAME = "com.ibm.team.build.internal.ui.BuildUIMessages"; //$NON-NLS-1$

    private BuildUIMessages() {
        // Intentionally Empty
    }

    static {
        // initialize resource bundle
        NLS.initializeMessages(BUNDLE_NAME, BuildUIMessages.class);

        // Instantiate an instance to get around code coverage limitation that
        // it can't currently exclude intentionally private constructors
        new BuildUIMessages();
    }

    /**
     * Name of the group for dialog notification settings.
     */
    public static String BuildPreferencePage_NotificationDialogGroupName;

    /**
     * Message for a read only property dialog.
     */
    public static String BuildPropertyControl_READ_ONLY_MESSAGE;

    /**
     * Title for a read only property dialog.
     */
    public static String BuildPropertyControl_READ_ONLY_TITLE;

    /**
     * Message indicating a required property.
     */
    public static String BuildPropertyControl_REQUIRED_PROPERTIES_MESSAGE;

    /**
     * Title indicating a required property.
     */
    public static String BuildPropertyControl_REQUIRED_PROPERTIES_TITLE;

    /**
     * Label for the string property kind.
     */
    public static String BuildPropertyControl_STRING_PROPERTY_KIND_LABEL;
    
    /**
     * Title for warning message indicating a pasted property was ignored.
     */
    public static String BuildPropertyControl_IGNORED_PASTED_PROPERTY_TITLE;
    
    /**
     * Warning message indicating a pasted property was ignored.
     */
    public static String BuildPropertyControl_IGNORED_PASTED_PROPERTY_MESSAGE;
    
    /**
     * Title for warning message indicating that a property was overwritten during the paste process.
     */
    public static String BuildPropertyControl_OVERWRITTEN_PROPERTY_TITLE;

    /**
     * Warning message indicating that a property was overwritten during the paste process.
     */
    public static String BuildPropertyControl_OVERWRITTEN_PROPERTY_MESSAGE;
    
    /**
     * Toggle text asking whether the user wants to ignore a dialog in the future
     */
    public static String BuildPropertyControl_TOGGLE_IGNORE_DIALOG_MESSAGE;

    /**
     * Message stating that an error occurred resolving the team repository.
     */
    public static String BuildItemHyperlinkHandler_ERROR_RESOLVING_REPOSITORY;

    /**
     * Message in {@link Status} if the URI does not have a valid repository
     * address.
     */
    public static String BuildResultHyperlinkHandler_BAD_URI_BAD_REPOSITORY;

    /**
     * Message in {@link Status} if the URI does not contain a build result item
     * id.
     */
    public static String BuildResultHyperlinkHandler_BAD_URI_NO_BUILD_RESULT;

    /**
     * Message in {@link Status} if the URI does not contain a valid
     * {@link Location}.
     */
    public static String BuildResultHyperlinkHandler_BAD_URI_NO_LOCATION;

    /**
     * Name column
     */
    public static String BuildPropertyControl_NAME;

    /**
     * Value volumn
     */
    public static String BuildPropertyControl_VALUE;

    /**
     * Add dialog title
     */
    public static String BuildPropertyControl_ADD_TITLE;

    /**
     * Text for the copy action in the build property control context menu
     */
    public static String BuildPropertyControl_COPY_ACTION_TEXT;

    /**
     * Edit dialog title
     */
    public static String BuildPropertyControl_EDIT_TITLE;

    /**
     * Description column
     */
    public static String BuildPropertyControl_DESCRIPTION;

    /**
     * Description label text
     */
    public static String BuildPropertyControl_DESCRIPTION_LABEL;

    /**
     * Description label text when multiple properties are selected.
     */
    public static String BuildPropertyControl_MULTIPLE_PROPERTIES_SELECTED_DESCRIPTION;

    /**
     * Description label text when multiple properties are selected.
     */
    public static String BuildPropertyControl_DESCRIPTION_LABEL_NO_DESCRIPTION;

    /**
     * Description label text when no properties are selected.
     */
    public static String BuildPropertyControl_NO_PROPERTY_SELECTED_DESCRIPTION;

    /**
     * Text for the paste action in the build property control context menu
     */
    public static String BuildPropertyControl_PASTE_ACTION_TEXT;

    /**
     * Pending message.
     */
    public static String PendingUpdateAdapter_PENDING;

    /**
     * The initial name of the job that opens the build result editor
     */
    public static String BuildUISupport_OPEN_BUILD_RESULT_EDITOR_JOB_TITLE;

    /**
     * The name that the job that opens the build result editor is changed to on
     * error.
     */
    public static String BuildUISupport_OPEN_BUILD_RESULT_EDITOR_JOB_TITLE_ON_ERROR;

    /**
     * Message indicating that an exception was thrown while opening the Build
     * Result Editor
     */
    public static String BuildUISupport_OPEN_BUILD_RESULT_EDITOR_EXCEPTION_MESSAGE;

    /**
     * Initial title of the job opening the build result
     */
    public static String BuildUISupport_OPEN_BUILD_RESULT_JOB_TITLE;

    /**
     * Title of the job that fetches the build result for
     * BuildResultLabelProvider
     */
    public static String BuildResultLabelProvider_FETCH_BUILD_RESULT_JOB_TITLE;

    /**
     * Message indicating an error fetching the build result
     */
    public static String BuildResultLabelProvider_FETCH_ERROR;

    /**
     * Title of the job that fetches the build definition for
     * BuildDefinitionLabelProvider
     */
    public static String BuildDefinitionLabelProvider_FETCH_BUILD_DEFINITION_JOB_TITLE;

    /**
     * The label format for a BuildDefinition with repository name.
     */
    public static String BUILD_DEFINITION_LABEL_WITH_REPOSITORY;

    /**
     * The label format for a BuildResult
     */
    public static String BUILD_RESULT_LABEL;

    /**
     * The label format for a BuildResult hyperlink
     */
    public static String BUILD_RESULT_HTML_LABEL;

    /**
     * The label separator for a BuildResult hyperlink
     */
    public static String BUILD_RESULT_HTML_LABEL_SEPARATOR;

    /**
     * Text to use for a build result hyperlink when an error occurred resolving
     * the build result.
     */
    public static String BUILD_RESULT_HYPERLINK_ERROR;

    /**
     * Text to use in for a deleted build result
     */
    public static String BUILD_RESULT_LABEL_DELETED;

    /**
     * Text to use for an unlabeled build result
     */
    public static String BUILD_RESULT_LABEL_UNLABELED;

    /**
     * Text to use for build result hyperlink details when an error occurred
     * resolving the build result.
     */
    public static String BUILD_RESULT_ERROR_HYPERLINK_DETAIL;

    /**
     * Text to use for build result hyperlink details when the build result is
     * deleted.
     */
    public static String BUILD_RESULT_DELETED_HYPERLINK_DETAIL;

    /**
     * Text to use in for a deleted build definition
     */
    public static String BUILD_DEFINITION_LABEL_DELETED;

    /**
     * Add button label "Add..."
     */
    public static String ADD_BUTTON_LABEL_WITH_ELLIPSIS;

    /**
     * Remove button label "Remove"
     */
    public static String REMOVE_BUTTON_LABEL;

    /**
     * Edit button
     */
    public static String EDIT_BUTTON_LABEL_WITH_ELLIPSIS;

    /**
     * Details for the URIReference of a hyperlink.
     */
    public static String BuildResultHyperlinkHandler_URIReferenceDetails;

    /**
     * Error message indicating an invalid state was specified for a build
     * activity.
     */
    public static String BuildCompositeImageRegistry_INVALID_STATE_FOR_BUILD_ACTIVITY;
    
    /**
     * String used in place of a project area name, if the project area name
     * cannot be found.
     */
    public static String UNKNOWN_PROJECT_AREA;
    
    /**
     * The label a table should use to display a build item and its project area
     * name.
     */
    public static String EXTENDED_LABEL;
    
    /**
     * Working directory control label
     */
    public static String MicrosoftBuildConfigurationEditor_WORKING_DIR_LABEL;
    
    /**
     * Working directory control description
     */
    public static String MicrosoftBuildConfigurationEditor_WORKING_DIR_DESC;
    
    /**
     * Solution file label
     */
    public static String MicrosoftBuildConfigurationEditor_SOLUTION_LABEL;
    
    /**
     * Solution file label
     */
    public static String MicrosoftBuildConfigurationEditor_SOLUTION_DESC;
    /**
     * Configuration label
     */
    public static String MicrosoftBuildConfigurationEditor_CONFIG_LABEL;
    /**
     * Configuration label description
     */
    public static String MicrosoftBuildConfigurationEditor_CONFIG_DESC;
    
    /**
     * Build type label
     */
    public static String MicrosoftBuildConfigurationEditor_BUILD_TYPE_LABEL;
    /**
     * Build type label description
     */
    public static String MicrosoftBuildConfigurationEditor_BUILD_TYPE_DESC;
    
    /**
     * Build log file 
     */
    public static String MicrosoftBuildConfigurationEditor_BUILD_OUTPUT_LABEL;
    
    /**
     * Build output control description
     */
    public static String MicrosoftBuildConfigurationEditor_BUILD_OUTPUT_DESC; 
    
    /**
     * Additional build commands 
     */
    public static String MicrosoftBuildConfigurationEditor_BUILD_ADDITIONAL_ARG_LABEL;
    
    /**
     * Additional build command control description
     */
    public static String MicrosoftBuildConfigurationEditor_BUILD_ADDITIONAL_ARG_DESC;
    
    /**
     * Add Build item information description
     */
    public static String MicrosoftBuildConfigurationEditor_ADD_BUILD_ITEM_DESC;
    
    /**
     * Edit Build item information description
     */
    public static String MicrosoftBuildConfigurationEditor_EDIT_BUILD_ITEM_DESC;

    /**
     * Add build item
     */
    public static String MicrosoftBuildConfigurationEditor_ADD_BUTTON_LABEL_WITH_ELLIPSIS;
    
    /**
     * Remove build item
     */
    public static String MicrosoftBuildConfigurationEditor_REMOVE_BUTTON_LABEL;

    /**
     * Edit build item
     */
    public static String MicrosoftBuildConfigurationEditor_EDIT_BUTTON_LABEL_WITH_ELLIPSIS;
    
    /**
     * Move the build item up
     */
    public static String MicrosoftBuildConfigurationEditor_MOVEUP_BUTTON_LABEL;
    
    /**
     * Move the build item down
     */
    public static String MicrosoftBuildConfigurationEditor_MOVEDOWN_BUTTON_LABEL;
    
    /**
     * Continue to build remaining items
     */
    public static String MicrosoftBuildConfigurationEditor_CONTINUE_BUILD_LABEL;

    /**
     * Preference check-box message for warning when build properties could not not be pasted
     */
    public static String NotificationDialogPreferencePage_IGNORED_PASTED_BUILD_PROPERTY_PREFERENCE_MESSAGE;
    
    /**
     * Preference check-box message for message when build properties are overwritten during pasting
     */
    public static String NotificationDialogPreferencePage_OVERWRITTEN_BUILD_PROPERTY_PREFERENCE_MESSAGE;
    
    /**
     * Preference check-box message for showing build folders in the build
     * domain.
     */
    public static String NotificationDialogPreferencePage_SHOW_BUILD_FOLDERS_PROPERTY_PREFERENCE_MESSAGE;
}
