/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui;

/**
 * Ids for context sensitive help.
 */
public interface IHelpContextIds {

    /**
     * Common prefix used by all help ids from this plugin
     */
    public static final String PREFIX = "com.ibm.team.build.ui"; //$NON-NLS-1$

    /*
     * VIEWS
     */

    /**
     * Help id for Builds view
     */
    public static final String HELP_CONTEXT_BUILDS_VIEW = PREFIX + ".view_builds"; //$NON-NLS-1$

    /*
     * NEW BUILD DEFINITION WIZARD
     */

    /**
     * Help id for the New Build Definition Wizard. This is what will show if a
     * particular wizard page does not set its own help context id.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_WIZARD = PREFIX + ".wizard_builddefinitiongeneral"; //$NON-NLS-1$

    /**
     * Help id for the Additional Configuration page of the New Build Definition
     * wizard.
     */
    public static final String HELP_CONTEXT_ADDITIONAL_CONFIGURATION_WIZARD_PAGE = PREFIX + ".wizard_additionalconfig"; //$NON-NLS-1$

    /**
     * Help id for the Post Build page of the New Build Definition wizard.
     */
    public static final String HELP_CONTEXT_POST_BUILD_WIZARD_PAGE = PREFIX + ".wizard_postbuild"; //$NON-NLS-1$

    /**
     * Help id for the Pre Build page of the New Build Definition wizard.
     */
    public static final String HELP_CONTEXT_PRE_BUILD_WIZARD_PAGE = PREFIX + ".wizard_prebuild"; //$NON-NLS-1$

    /**
     * Help id for the Team Area Selection page of the New Build Definition
     * wizard.
     */
    public static final String HELP_CONTEXT_TEAM_AREA_SELECTION_WIZARD_PAGE = PREFIX + ".wizard_teamareaselection"; //$NON-NLS-1$

    /**
     * Help id for the Build Template Selection page of the New Build Definition
     * wizard.
     */
    public static final String HELP_CONTEXT_BUILD_TEMPLATE_SELECTION_WIZARD_PAGE = PREFIX + ".wizard_templateselection"; //$NON-NLS-1$

    /*
     * NEW BUILD ENGINE WIZARD
     */

    /**
     * Help id for the Team Area Selection page of the New Build Engine wizard.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_ENGINE_TEAM_AREA_SELECTION_WIZARD_PAGE = PREFIX
            + ".wizard_engineteamareaselection"; //$NON-NLS-1$

    /**
     * Help id for the Build Engine Template Selection page of the New Build
     * Engine wizard.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_ENGINE_TEMPLATE_SELECTION_WIZARD_PAGE = PREFIX
            + ".wizard_enginetemplateselection"; //$NON-NLS-1$

    /*
     * BUILD DEFINITION EDITOR PAGES
     */

    /**
     * Help id for the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR = PREFIX + ".editor_builddefinition"; //$NON-NLS-1$

    /**
     * Help id for the "Ant" page (tab) of the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_ANT_PAGE = PREFIX + ".editor_builddefinitionant"; //$NON-NLS-1$

    /**
     * Help id for the "Command Line" page (tab) of the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_COMMAND_LINE = PREFIX
            + ".editor_builddefinitioncommand"; //$NON-NLS-1$

    /**
     * Help id for the "Expected Contributions" page (tab) of the Build
     * Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_EXPECTED_CONTRIBUTIONS = PREFIX
            + ".editor_builddefinitioncontributions"; //$NON-NLS-1$

    /**
     * Help id for the "Overview" page (tab) of the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_GENERAL = PREFIX + ".editor_builddefinitiongeneral"; //$NON-NLS-1$

    /**
     * Help id for the "Jazz Source Control" page (tab) of the Build Definition
     * Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_SCM = PREFIX + ".editor_builddefinitionjazzscm"; //$NON-NLS-1$

    /**
     * Help id for the "ECJ Publishing" (tab) of the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_JDT = PREFIX + ".editor_builddefinitionjdt"; //$NON-NLS-1$

    /**
     * Help id for the "JUnit Publishing" page (tab) of the Build Definition
     * Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_JUNIT = PREFIX + ".editor_builddefinitionjunit"; //$NON-NLS-1$

    /**
     * Help id for the "Properties" page (tab) of the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_PROPERTIES = PREFIX
            + ".editor_builddefinitionproperties"; //$NON-NLS-1$

    /**
     * Help id for the "Schedule" page (tab) of the Build Definition Editor.
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_SCHEDULE = PREFIX
            + ".editor_builddefinitionschedule"; //$NON-NLS-1$

    /**
     * Help id for the "Email Notification" page (tab) of the Build Definition
     * Editor.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_EMAIL_NOTIFICATION = PREFIX
            + ".editor_builddefinitionemailnotification"; //$NON-NLS-1$

    /**
     * Help id for the "Maven" page (tab) of the Build Definition Editor.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_MAVEN = PREFIX + ".editor_builddefinitionmaven"; //$NON-NLS-1$

    /**
     * Help id for the "Microsoft Build" page (tab) of the Build Definition
     * Editor.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_MICROSOFT_BUILD = PREFIX
            + ".editor_builddefinitionmicrosoftbuild"; //$NON-NLS-1$

    /**
     * Help id for the "MSTest Configuration" page (tab) of the Build Definition
     * Editor.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_MSTEST = PREFIX + ".editor_builddefinitionmstest"; //$NON-NLS-1$

    /**
     * Help id for the "NUnit Configuration" page (tab) of the Build Definition
     * Editor.
     * 
     * @since 0.8
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_NUNIT = PREFIX + ".editor_builddefinitionnunit"; //$NON-NLS-1$

    /**
     * Help id for the "Post-build Deliver" [a.k.a. 'Auto-Deliver'] (tab) of the
     * Build Definition Editor.
     * 
     * @since 0.8.1
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_AUTO_DELIVER = PREFIX + ".editor_builddefinitionpostbuilddeliver"; //$NON-NLS-1$

    /**
     * Help id for the "Post-build Deploy" page (tab) of the Build Definition Editor.
     * 
     * @since 0.9.5
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_EDITOR_UDEPLOY = PREFIX
            + ".editor_builddefinitionudeploy"; //$NON-NLS-1$

    /*
     * BUILD DEFINITION EDITOR PAGES
     */

    /**
     * Help id for the Build Engine Editor.
     */
    public static final String HELP_CONTEXT_BUILD_ENGINE_EDITOR = PREFIX + ".editor_buildengine"; //$NON-NLS-1$

    /*
     * BUILD RESULT EDITOR PAGES
     */

    /**
     * Help id for the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR = PREFIX + ".editor_buildresult"; //$NON-NLS-1$

    /**
     * Help id for the Artifacts page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_ARTIFACTS_PAGE = PREFIX
            + ".editorpage_buildresultartifact"; //$NON-NLS-1$

    /**
     * Help id fort the Links page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_LINKS_PAGE = PREFIX + ".editorpage_buildresultlink"; //$NON-NLS-1$

    /**
     * Help id for the Logs page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_LOGS_PAGE = PREFIX + ".editorpage_buildresultlog"; //$NON-NLS-1$

    /**
     * Help id for the Activities page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_ACTIVITIES_PAGE = PREFIX
            + ".editorpage_buildresultactivities"; //$NON-NLS-1$

    /**
     * Help id for the Properties page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_PROPERTIES_PAGE = PREFIX
            + ".editorpage_buildresultproperties"; //$NON-NLS-1$

    /**
     * Help id for the Summary page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_SUMMARY_PAGE = PREFIX + ".editor_buildresultsummary"; //$NON-NLS-1$

    /**
     * Help id for the Compilation page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_COMPILE_PAGE = PREFIX + ".editor_buildresultcompile"; //$NON-NLS-1$

    /**
     * Help id for the JUnit page (tab) of the Build Result Editor.
     */
    public static final String HELP_CONTEXT_BUILD_RESULT_EDITOR_JUNIT_PAGE = PREFIX + ".editor_buildresultjunit"; //$NON-NLS-1$

    /* DIALOGS */

    /**
     * Help id for the Request build dialog
     */
    public static final String HELP_CONTEXT_REQUEST_BUILD_DIALOG = PREFIX + ".dialog_requestbuild"; //$NON-NLS-1$

    /**
     * Help id for the Select Build Definition dialog
     */
    public static final String HELP_CONTEXT_BUILD_DEFINITION_DIALOG = PREFIX + ".dialog_builddefinition"; //$NON-NLS-1$

    /**
     * Help id for the Build Property dialog.
     */
    public static final String HELP_CONTEXT_BUILD_PROPERTY_DIALOG = PREFIX + ".dialog_buildproperty"; //$NON-NLS-1$
    
    /**
     * Help id for the Microsoft Build Item dialog.
     */
    public static final String HELP_CONTEXT_MS_BUILD_ITEM_DIALOG = PREFIX + ".dialog_msbuilditem"; //$NON-NLS-1$


    /**
     * Help id for the dialog that prompts the user for a tag to search for
     */
    public static final String HELP_CONTEXT_TAG_DIALOG = PREFIX + ".dialog_tag"; //$NON-NLS-1$

    /**
     * Help id for the Configure My Builds Menu dialog
     */
    public static final String HELP_CONTEXT_CONFIGURE_MY_BUILDS_DIALOG = PREFIX + ".dialog_configuremybuilds"; //$NON-NLS-1$

    /**
     * Help id for the Edit Configuration dialog.
     */
    public static final String HELP_CONTEXT_EDIT_CONFIGURATION_DIALOG = PREFIX + ".dialog_editconfiguration"; //$NON-NLS-1$    

    /**
     * Help id for the Builds section of Team Central
     */
    public static final String HELP_CONTEXT_BUILDS_SECTION = PREFIX + ".teamcentral"; //$NON-NLS-1$

    /* ACTIONS */

}
