/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IWorkbenchPartSite;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;

/**
 * Abstract base class for actions that need to interact with repository files
 * associated with build contributions.
 */
public abstract class AbstractRepositoryFileAction extends Action implements IAction {

    private final IWorkbenchPartSite fPartSite;
    private final ITeamRepository fTeamRepository;
    private final String fFileName;
    private final IContent fContent;

    /**
     * Create an AbstractRepositoryFileAction.
     * 
     * @param partSite
     *            The {@link IWorkbenchPartSite} used to access action bars.
     * @param teamRepository
     *            The {@link ITeamRepository} in which the contribution is
     *            stored.
     * @param fileName
     *            The fileName to use.
     * @param content
     *            The content of the file.
     */
    public AbstractRepositoryFileAction(IWorkbenchPartSite partSite, ITeamRepository teamRepository, String fileName,
            IContent content) {
        fPartSite = partSite;
        fTeamRepository = teamRepository;
        fFileName = fileName;
        fContent = content;
    }

    /**
     * Allow access to the file name of the file to be downloaded.
     * 
     * @return The file name of the file to be downloaded.
     */
    public String getFileName() {
        return fFileName;
    }

    /**
     * Allow access to the content of the file to be downloaded.
     * 
     * @return The content of the file to be downloaded.
     */
    public IContent getContent() {
        return fContent;
    }

    /**
     * Allow access to the current part
     * 
     * @return the current part
     */
    public IWorkbenchPartSite getPartSite() {
        return fPartSite;
    }

    /**
     * Allow access to the repository
     * 
     * @return the repository the file will be downloaded from
     */
    public ITeamRepository getTeamRepository() {
        return fTeamRepository;
    }

}
