/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import java.text.MessageFormat;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.IBuildProperty;
import com.ibm.team.build.internal.ui.helper.ClipboardHelper;

/**
 * Copies build property to clipboard in .properties format. For instance:
 * #description name = value
 * 
 * #description name = value
 */
public class CopyBuildPropertyAsPropertiesFormatActionDelegate extends ActionDelegate implements IActionDelegate {

    /**
     * Text for the line that has the name and value pair.
     */
    private static final String NAME_VALUE_LINE = "{0} = {1}"; //$NON-NLS-1$
    /**
     * Text for the line that shows the description.
     */
    private static final String DESCRIPTION_LINE = "# {0}"; //$NON-NLS-1$
    private static final String LINE_SEPARATOR = System.getProperty("line.separator"); //$NON-NLS-1$

    private List fSelectedElements;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void run(IAction action) {
        if (fSelectedElements != null && !fSelectedElements.isEmpty()) {
            StringBuffer fBuffer = new StringBuffer();
            Iterator<IBuildProperty> iter = fSelectedElements.iterator();
            while (iter.hasNext()) {
                Object next = iter.next();
                if (next instanceof IBuildProperty) {
                    IBuildProperty currentProperty = (IBuildProperty) next;
                    String description = currentProperty.getDescription();
                    if (description != null && !description.equals("")) { //$NON-NLS-1$
                        fBuffer.append(MessageFormat.format(DESCRIPTION_LINE, description));
                        fBuffer.append(LINE_SEPARATOR);
                    }
                    fBuffer.append(MessageFormat.format(NAME_VALUE_LINE, new Object[] { currentProperty.getName(),
                            currentProperty.getValue() }));
                    if (iter.hasNext()) {
                        fBuffer.append(System.getProperty("line.separator")); //$NON-NLS-1$
                        fBuffer.append(System.getProperty("line.separator")); //$NON-NLS-1$
                    }
                }
            }
            ClipboardHelper.getDefault().copyTextToClipboard(fBuffer.toString(), Display.getCurrent());
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void selectionChanged(IAction action, ISelection selection) {
        if (selection instanceof IStructuredSelection) {
            fSelectedElements = ((IStructuredSelection) selection).toList();
        }
    }

}
