/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.internal.ui.editors.result.download.CopyToClipboardAction;
import com.ibm.team.build.internal.ui.editors.result.download.DownloadContributionNode;

/**
 * Action delegate for copying the link location of a file represented by a
 * {@link DownloadContributionNode}
 */
public class CopyDownloadLinkLocationActionDelegate extends ActionDelegate implements IEditorActionDelegate {

    private IStructuredSelection fSelection;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void selectionChanged(IAction action, ISelection selection) {
        if (selection instanceof IStructuredSelection) {
            fSelection = (IStructuredSelection) selection;
            if (action != null) {
                updateEnablement(action, fSelection);
            }
        }
    }

    protected void updateEnablement(IAction action, IStructuredSelection selection) {
        DownloadContributionNode node = getNode(selection);
        
        if (node != null) {
            action.setText(node.isExternalArtifact() ? BuildUIActionMessages.CopyToClipboardAction_LinkMenuText
                    : BuildUIActionMessages.CopyToClipboardAction_FileMenuText);
        }
    }

    protected DownloadContributionNode getNode(IStructuredSelection selection) {
        Object element = ((IStructuredSelection) selection).getFirstElement();
        if (element instanceof DownloadContributionNode) {
            return (DownloadContributionNode) element;
        }
        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActiveEditor(IAction action, IEditorPart targetEditor) {
        // do nothing
    }

    public void run(IAction action) {
        if (fSelection.getFirstElement() instanceof DownloadContributionNode) {
            DownloadContributionNode node = (DownloadContributionNode) fSelection.getFirstElement();
            runAction(new CopyToClipboardAction(node.getFileURL(), node.isExternalArtifact()));
        }
    }

    protected void runAction(IAction action) {
        action.run();
    }
}
