/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.model.IBuildDefinition;
import com.ibm.team.build.common.model.query.IBaseBuildResultQueryModel.IBuildResultQueryModel;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.common.query.IDataQuery;
import com.ibm.team.repository.common.query.IDataQueryPage;
import com.ibm.team.repository.common.query.IDataRow;

/**
 * Action to remove one or more a build definitions.
 */
public class DeleteBuildDefinitionActionDelegate extends AbstractRepositoryActionDelegate implements
        IObjectActionDelegate {

    protected List<IBuildDefinition> fBuildDefinitions = new ArrayList<IBuildDefinition>();

    public void selectionChanged(IAction action, ISelection selection) {
        super.selectionChanged(action, selection);

        fBuildDefinitions.clear();

        IStructuredSelection structuredSelection = (IStructuredSelection) selection;

        Iterator iterator = structuredSelection.iterator();
        while (iterator.hasNext()) {
            Object object = iterator.next();
            if (object instanceof IBuildDefinition) {
                fBuildDefinitions.add((IBuildDefinition) object);
            }
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getConfirmationMessage() {
        return BuildUIActionMessages.RemoveBuildDefinitionActionDelegate_1;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getJobLabel() {
        return BuildUIActionMessages.RemoveBuildDefinitionActionDelegate_0;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        progressMonitor.beginTask(BuildUIActionMessages.RemoveBuildDefinitionActionDelegate_0,
                fBuildDefinitions.size() * 2);

        try {
            for (IBuildDefinition buildDefinition : fBuildDefinitions) {
                ITeamRepository teamRepository = (ITeamRepository) buildDefinition.getOrigin();
                ITeamBuildClient teamBuildClient = ClientFactory.getTeamBuildClient(teamRepository);

                IBuildResultQueryModel queryModel = IBuildResultQueryModel.ROOT;
                IDataQuery query = IDataQuery.FACTORY.newInstance(queryModel);
                query.select(query.count());
                query.filter(queryModel.buildDefinition()._eq(query.newItemHandleArg()));
                IDataQueryPage page = teamBuildClient.queryData(query, new Object[] { buildDefinition }, 1,
                        new SubProgressMonitor(progressMonitor, 1));
                IDataRow row = page.getRow(0);

                if (row.getInt(0) > 0) {
                    openError(BuildUIActionMessages.DeleteBuildDefinitionActionDelegate_DELETE_PROBLEM, NLS.bind(
                            BuildUIActionMessages.DeleteBuildDefinitionActionDelegate_CANT_DELETE,
                            buildDefinition.getId()));

                    continue;
                }

                try {
                    teamBuildClient.delete(buildDefinition, new SubProgressMonitor(progressMonitor, 1));
                } catch (ItemNotFoundException exception) {
                    // the build definition was already deleted, do nothing
                }
            }
        } finally {
            progressMonitor.done();
        }
    }

    protected void openError(final String title, final String message) {
        Display.getDefault().asyncExec(new Runnable() {
            public void run() {
                MessageDialog.openError(getShell(), title, message);
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        // Nothing to do.
    }
}
