/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import com.ibm.team.build.client.ClientFactory;
import com.ibm.team.build.client.ITeamBuildClient;
import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Action delegate to remove one or more build engines from the repository.
 */
public class DeleteBuildEngineActionDelegate extends AbstractRepositoryActionDelegate implements IObjectActionDelegate {

    private ISelection fSelection;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        // Nothing to do.
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getConfirmationMessage() {
        return BuildUIActionMessages.DeleteBuildEngineActionDelegate_0;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected String getJobLabel() {
        return BuildUIActionMessages.DeleteBuildEngineActionDelegate_1;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        List<IBuildEngine> selectedEngines = new ArrayList<IBuildEngine>();
        IStructuredSelection selection = (IStructuredSelection) fSelection;
        Iterator iter = selection.iterator();
        while (iter.hasNext()) {
            Object element = iter.next();
            if (element instanceof IBuildEngine) {
                selectedEngines.add((IBuildEngine) element);
            }
        }

        removeEngines(selectedEngines, progressMonitor);
    }

    public void selectionChanged(IAction action, ISelection selection) {
        fSelection = selection;
    }

    private void removeEngines(List<IBuildEngine> engines, IProgressMonitor progressMonitor)
            throws TeamRepositoryException {

        progressMonitor.beginTask(BuildUIActionMessages.DeleteBuildEngineActionDelegate_1, engines.size());

        try {
            for (IBuildEngine engine : engines) {
                ITeamRepository teamRepository = (ITeamRepository) engine.getOrigin();
                ITeamBuildClient teamBuildClient = ClientFactory.getTeamBuildClient(teamRepository);
                try {
                    teamBuildClient.delete(engine, new SubProgressMonitor(progressMonitor, 1));
                } catch (ItemNotFoundException exception) {
                    // build engine was already deleted - do nothing
                }
            }
        } finally {
            progressMonitor.done();
        }
    }
}
