/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.common.model.IBuildEngine;
import com.ibm.team.build.internal.ui.BuildUIPlugin;
import com.ibm.team.build.internal.ui.IBuildUIConstants;
import com.ibm.team.build.internal.ui.editors.buildengine.BuildEngineEditorInput;
import com.ibm.team.build.internal.ui.jobs.TeamBuildJob;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Opens a build engine editor.
 */
public class EditBuildEngineActionDelegate extends ActionDelegate implements IObjectActionDelegate {

    private IStructuredSelection fSelection;
    private IWorkbenchPart fTargetPart;

    public void selectionChanged(IAction action, ISelection selection) {
        if (selection instanceof IStructuredSelection) {
            fSelection = (IStructuredSelection) selection;
        }
    }

    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fTargetPart = targetPart;
    }

    public void run(IAction action) {
        if (fSelection.getFirstElement() instanceof IBuildEngine) {
            IBuildEngine buildEngine = (IBuildEngine) fSelection.getFirstElement();
            run(buildEngine, fTargetPart.getSite().getPage());
        }
    }

    /**
     * Open the build engine editor on the given build engine.
     * 
     * @param buildEngine
     *            the build engine to edit
     * @param workbenchPage
     *            the workbench page
     */
    public static void run(final IBuildEngine buildEngine, final IWorkbenchPage workbenchPage) {

        String jobName = BuildUIActionMessages.EditBuildDefinitionActionDelegate_FETCHING_TEAM_AREA;
        Job job = new TeamBuildJob(jobName, true) {
            @Override
            protected IStatus runProtected(IProgressMonitor monitor) throws Exception {
                ITeamRepository teamRepository = (ITeamRepository) buildEngine.getOrigin();
                IProcessAreaHandle processAreaHandle = buildEngine.getProcessArea();

                final IProcessArea processArea = (IProcessArea) teamRepository.itemManager().fetchCompleteItem(
                        processAreaHandle, IItemManager.DEFAULT, monitor);

                Display.getDefault().asyncExec(new Runnable() {
                    public void run() {
                        handleAsyncExec(workbenchPage, buildEngine, processArea);
                    }
                });
                return Status.OK_STATUS;
            }
        };
        job.schedule();
    }

    private static void handleAsyncExec(IWorkbenchPage workbenchPage, IBuildEngine buildEngine, IProcessArea processArea) {
        if (!workbenchPage.getWorkbenchWindow().getShell().isDisposed()) {
            try {
                BuildEngineEditorInput input = new BuildEngineEditorInput(buildEngine, processArea,
                        (ITeamRepository) buildEngine.getOrigin());
                workbenchPage.openEditor(input, IBuildUIConstants.ID_BUILD_ENGINE_EDITOR);
            } catch (PartInitException exception) {
                BuildUIPlugin.log(exception);
            }
        }
    }
}
