/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.build.common.model.IBuildFolderHandle;
import com.ibm.team.build.internal.ui.dialogs.BuildFolderPropertiesDialog;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Action that fetches a build folder and opens a dialog to edit it.
 */
public class EditBuildFolderActionDelegate extends AbstractRepositoryActionDelegate implements IObjectActionDelegate {

    private IBuildFolderHandle fBuildFolderHandle;
    private IWorkbenchSite fSite;

    /**
     * Create the edit build folder action delegate.
     * 
     * @param site
     *            the <code>IWorkbenchSite</code> for the dialog.
     */
    public EditBuildFolderActionDelegate(IWorkbenchSite site) {
        fSite = site;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected String getConfirmationMessage() {
        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected void performAction(IProgressMonitor progressMonitor) throws TeamRepositoryException {
        PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
            @Override
            public void run() {
                if (!fSite.getShell().isDisposed()) {
                    BuildFolderPropertiesDialog.openEditFolderDialog(fSite.getShell(), fBuildFolderHandle);
                }
            }
        });
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    protected String getJobLabel() {
        return BuildUIActionMessages.EditBuildFolderActionDelegate_JOB_TITLE;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        // Do nothing.
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    @Override
    public void selectionChanged(IAction action, ISelection selection) {
        IStructuredSelection structuredSelection = (IStructuredSelection) selection;
        if (structuredSelection.size() == 1 && structuredSelection.getFirstElement() instanceof IBuildFolderHandle) {
            fBuildFolderHandle = (IBuildFolderHandle) structuredSelection.getFirstElement();
        }
    }
}
