/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.wizard.IWizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.internal.ui.domain.IActionIds;
import com.ibm.team.build.internal.ui.domain.Messages;
import com.ibm.team.build.internal.ui.wizards.buildengine.NewBuildEngineWizard;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Action delegate that opens the New Build Engine wizard.
 */
public class NewBuildEngineActionDelegate extends ActionDelegate implements IObjectActionDelegate {
    /**
     * The wizard dialog width
     */
    private static final int WIZARD_WIDTH = 500;

    /**
     * The wizard dialog height
     */
    private static final int WIZARD_HEIGHT = 500;

    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used by subclasses.
     */
    protected ITeamRepository fTeamRepository;

    /**
     * This field is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used by subclasses.
     */
    protected IWorkbenchPart fWorkbenchPart;

    private IProjectAreaHandle fProjectAreaHandle;

    /**
     * Get an action for opening the New Build Engine wizard.
     * 
     * @param projectAreaHandle
     *            The project area to use as context when selecting a team area.
     *            If null, the user must choose among all project areas when
     *            selecting a team area.
     * @param parent
     *            The parent context for the dialog.
     * @return An action for opening the New Build Engine wizard.
     */
    public Action getAction(final IProjectAreaHandle projectAreaHandle, final Shell parent) {
        return new Action() {
            {
                setId(IActionIds.NEW_BUILD_ENGINE_ID);
                setText(Messages.BuildDomainActionHelper_NEW_BUILD_ENGINE);
            }
            
            @Override
            public void run() {
                ITeamRepository teamRepository = null;
                if (projectAreaHandle != null) {
                    teamRepository = (ITeamRepository) projectAreaHandle.getOrigin();
                }
                NewBuildEngineActionDelegate.this.run(projectAreaHandle, teamRepository, parent);
            }
        };
    }

    private void run(IProjectAreaHandle projectAreaHandle, ITeamRepository teamRepository, Shell parent) {
        NewBuildEngineWizard wizard = null;
        if (projectAreaHandle != null) {
            wizard = new NewBuildEngineWizard(projectAreaHandle);
        } else if (teamRepository != null) {
            wizard = new NewBuildEngineWizard(teamRepository);
        } else {
            wizard = new NewBuildEngineWizard();
        }

        WizardDialog dialog = getDialog(parent, wizard);
        dialog.create();
        dialog.getShell().pack();
        dialog.getShell().setSize(Math.max(WIZARD_WIDTH, dialog.getShell().getSize().x), WIZARD_HEIGHT);
        dialog.open();
    }

    public void run(IAction action) {
        IWorkbenchPage page = fWorkbenchPart.getSite().getPage();
        if (page != null) {
            run(fProjectAreaHandle, fTeamRepository, page.getWorkbenchWindow().getShell());
        }
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fWorkbenchPart = targetPart;
    }

    /**
     * This method is <code>protected</code> instead of <code>private</code>
     * to enable testing. It is not intended to be used or overridden by
     * subclasses.
     */
    protected WizardDialog getDialog(Shell parent, IWizard wizard) {
        return new WizardDialog(parent, wizard);
    }

}
