/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import org.eclipse.ui.IWorkbenchPartSite;

import com.ibm.team.build.internal.ui.helper.RepositoryFileHelper;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContent;

/**
 * Action to display the content associated with a build contribution.
 */
public class OpenRepositoryFileAction extends AbstractRepositoryFileAction {

    /**
     * Create an action to open a file from the repository in an editor.
     * 
     * @param partSite
     *            The {@link IWorkbenchPartSite} used to access action bars.
     * @param teamRepository
     *            The {@link ITeamRepository} in which the contribution is
     *            stored.
     * @param fileName
     *            The fileName to use.
     * @param content
     *            The content of the file.
     */
    public OpenRepositoryFileAction(IWorkbenchPartSite partSite, ITeamRepository teamRepository, String fileName,
            IContent content) {
        super(partSite, teamRepository, fileName, content);
    }

    /**
     * {@inheritDoc} Download file associated with selected content to
     * the temp directory and open an editor to diplay the file contents.
     */
    public void run() {
        RepositoryFileHelper.openRepositoryFile(getFileName(), getContent(), getTeamRepository(), getPartSite());
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getText() {
        return BuildUIActionMessages.OpenFileAction_OPEN;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getToolTipText() {
        return BuildUIActionMessages.OpenFileAction_OPEN_FILE_IN_EDITOR_TOOLTIP;
    }
}
