/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import java.net.URL;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IWorkbenchPartSite;

import com.ibm.team.build.internal.ui.helper.URLHelper;

/**
 * This action implements downloading and opening a file specified by a
 * {@link URL}.
 */
public class OpenURLAction extends Action implements IAction {

    private String fUrl;
    private IWorkbenchPartSite fPartSite;
    private boolean fOpenInBrowser;

    /**
     * Create an action to download and open a file specified by a URL.
     * 
     * @param url
     *            The url to download and open.
     * @param partSite
     *            The {@link IWorkbenchPartSite} containing the URL.
     * @param openInBrowser
     *            If true, pass the URL directly to the browser regardless of
     *            protocol.
     */
    public OpenURLAction(String url, IWorkbenchPartSite partSite, boolean openInBrowser) {
        fUrl = url;
        fPartSite = partSite;
        fOpenInBrowser = openInBrowser;
    }

    /**
     * {@inheritDoc} Download file associated with selected content to the temp
     * directory and open an editor to diplay the file contents.
     */
    public void run() {
        openURL(fUrl, fPartSite, fOpenInBrowser);
    }

    protected void openURL(String url, IWorkbenchPartSite partSite, boolean openInBrowser) {
        URLHelper.openURL(fUrl, openInBrowser, partSite);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getText() {
        return BuildUIActionMessages.OpenFileAction_OPEN;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getToolTipText() {
        return BuildUIActionMessages.OpenURLAction_OPEN_URL_TOOLTIP;
    }
}