/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionDelegate;

import com.ibm.team.build.internal.ui.editors.result.download.DownloadContributionNode;

/**
 * Action delegate for saving an artifact represented by a
 * {@link DownloadContributionNode}
 */
public class SaveDownloadActionDelegate extends ActionDelegate implements IObjectActionDelegate {

    private IStructuredSelection fSelection;
    private IWorkbenchPart fTargetPart;

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void selectionChanged(IAction action, ISelection selection) {
        if (selection instanceof IStructuredSelection) {
            fSelection = (IStructuredSelection) selection;
            if (fSelection.size() == 1 && action != null) {
                updateEnablement(action, fSelection);
            }
        }
    }

    protected void updateEnablement(IAction action, IStructuredSelection selection) {
        DownloadContributionNode node = getNode(selection);

        if (node != null) {
            action.setEnabled(node.isExternalArtifact() || node.getContribution().getExtendedContributionData() != null);
        }
    }

    protected DownloadContributionNode getNode(IStructuredSelection selection) {
        Object element = ((IStructuredSelection) selection).getFirstElement();
        if (element instanceof DownloadContributionNode) {
            return (DownloadContributionNode) element;
        }
        return null;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart) {
        fTargetPart = targetPart;
    }

    public void run(IAction action) {
        if (fSelection.getFirstElement() instanceof DownloadContributionNode) {
            IAction saveAction = null;

            DownloadContributionNode node = (DownloadContributionNode) fSelection.getFirstElement();
            if (node.isExternalArtifact()) {
                saveAction = new SaveURLAction(node.getFileURL(), fTargetPart.getSite().getShell());
            } else {
                saveAction = new SaveRepositoryFileAction(fTargetPart.getSite(), node.getRepository(),
                        node.getFileName(), node.getContribution().getExtendedContributionData());
            }

            runAction(saveAction);
        }
    }

    protected void runAction(IAction action) {
        action.run();
    }
}
