/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.actions;

import java.net.URL;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.build.internal.ui.helper.URLHelper;

/**
 * This action implements downloading and saving a file specified by a
 * {@link URL}.
 */
public class SaveURLAction extends Action implements IAction {

    private String fUrl;
    private Shell fShell;

    /**
     * Create an action to download and save a file specified by a URL.
     * 
     * @param url
     *            The url to download and save.
     * @param shell
     *            The {@link Shell} in which to display progress information.
     */
    public SaveURLAction(String url, Shell shell) {
        fUrl = url;
        fShell = shell;
    }

    /**
     * {@inheritDoc} Download file associated with selected content to
     * the temp directory and open an editor to diplay the file contents.
     */
    public void run() {
        saveURLToFile(fUrl, fShell);
    }

    protected void saveURLToFile(String url, Shell shell) {
        URLHelper.saveURLToFile(fUrl, fShell);
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getText() {
        return BuildUIActionMessages.SaveURLAction_SAVE_LINK_AS;
    }

    /*
     * (non-Javadoc) Intentionally not documented. See parent.
     */
    public String getToolTipText() {
        return BuildUIActionMessages.SaveURLAction_SAVE_AS_TOOLTIP;
    }
}