/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.build.internal.ui.actions;

import java.util.List;

import org.eclipse.jface.action.Action;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IWorkbenchPage;

import com.ibm.team.build.common.model.IBuildDefinitionHandle;
import com.ibm.team.build.internal.ui.domain.BuildDomainQueryNode;
import com.ibm.team.build.internal.ui.domain.IActionIds;
import com.ibm.team.build.internal.ui.domain.Messages;
import com.ibm.team.build.internal.ui.query.BuildQueryByDefinition;
import com.ibm.team.build.internal.ui.views.query.AdaptableBuildQueryRow;
import com.ibm.team.build.internal.ui.views.query.BuildQueryView;
import com.ibm.team.repository.client.ITeamRepository;

/**
 * Action that shows the build results for a build definition in the Builds
 * view.
 */
public final class ShowBuildResultsAction extends Action {
    
    private final List<BuildDomainQueryNode> fBuildDefinitionQueryNodes;
    
    private final List<IBuildDefinitionHandle> fBuildDefinitionHandles;
    private IWorkbenchPage fWorkbenchPage;

    /**
     * Create a new action.
     * @param buildDefinitionQueryNodes List of {@link BuildDomainQueryNode} that this action can u
     * @param buildDefinitionHandles List of {@link IBuildDefinitionHandle} that will be used in the query that this action will display.
     * @param workbenchPage 
     * 
     */
    public ShowBuildResultsAction(List<BuildDomainQueryNode> buildDefinitionQueryNodes,
            List<IBuildDefinitionHandle> buildDefinitionHandles, IWorkbenchPage workbenchPage) {
        super(Messages.BuildDomainActionHelper_SHOW_BUILDS);
        fBuildDefinitionQueryNodes = buildDefinitionQueryNodes;
        fBuildDefinitionHandles = buildDefinitionHandles;
        fWorkbenchPage = workbenchPage;
    }

    @Override
    public void run() {
        ITeamRepository teamRepository = (ITeamRepository) fBuildDefinitionHandles.get(0).getOrigin();

        String name = fBuildDefinitionQueryNodes.get(0).getLabel();
        if (fBuildDefinitionHandles.size() > 1) {
            name = NLS.bind(Messages.BuildDomainActionHelper_QUERY_NAME_MULTIPLE, fBuildDefinitionHandles.size());
        }
        BuildQueryByDefinition query = new BuildQueryByDefinition(name, null, AdaptableBuildQueryRow.Factory,
                fBuildDefinitionHandles, teamRepository);
        BuildQueryView.displayQuery(query, fWorkbenchPage);

    }

    @Override
    public String getId() {
        return IActionIds.SHOW_BUILDS_SELECTED_TYPES;
    }
}