/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.build.internal.ui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;

/**
 * A dialog for choosing a build property kind.
 */
public class BuildPropertyKindSelectionDialog extends Dialog {
    protected String fTitle;
    private String[] fPropertyKinds;
    private Table fTable;
    protected int fSelectedKind = -1;
    protected TableViewer fTableViewer;

    /**
     * Create a dialog to add or edit a build property.
     * 
     * @param parentShell
     *            the parent <code>Shell</code>; never <code>null</code>.
     * @param title
     *            the dialog title; never <code>null</code>.
     * @param propertyKinds
     *            The available property kinds.
     */
    public BuildPropertyKindSelectionDialog(Shell parentShell, String title, String[] propertyKinds) {
        super(parentShell);
        setShellStyle(getShellStyle() | SWT.RESIZE);
        fTitle = title;
        fPropertyKinds = propertyKinds;
    }

    /**
     * Get the index of the selected property kind. The index corresponds to the
     * <code>propertyKinds</code> array that this object was constructed with.
     * 
     * @return The index of the selected property kind.
     */
    public int getSelectedKind() {
        return fSelectedKind;
    }

    protected Control createDialogArea(Composite parent) {
        Composite composite = (Composite) super.createDialogArea(parent);
        composite.setLayout(new GridLayout(1, false));
        GridData gridData = new GridData(GridData.FILL_BOTH);
        gridData.widthHint = 300;
        composite.setLayoutData(gridData);

        Label label = new Label(composite, SWT.WRAP);
        gridData = new GridData(GridData.FILL_HORIZONTAL);
        label.setLayoutData(gridData);
        label.setText(BuildUIDialogMessages.BuildPropertyKindSelectionDialog_DESCRIPTION);

        fTableViewer = new TableViewer(composite, SWT.SINGLE | SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER);
        fTable = fTableViewer.getTable();
        gridData = new GridData(GridData.FILL_BOTH);
        gridData.heightHint = 8 * fTable.getItemHeight();
        fTable.setLayoutData(gridData);

        fTableViewer.setContentProvider(new ArrayContentProvider());
        fTableViewer.addSelectionChangedListener(getSelectionListener());
        fTableViewer.addDoubleClickListener(getDoubleClickListener());
        fTableViewer.setInput(fPropertyKinds);
        fTable.select(0);
        fSelectedKind = 0;

        applyDialogFont(composite);

        return composite;
    }

    protected IDoubleClickListener getDoubleClickListener() {
        return new IDoubleClickListener() {
            public void doubleClick(DoubleClickEvent event) {
                fSelectedKind = fTable.getSelectionIndex();
                okPressed();
            }
        };
    }

    protected ISelectionChangedListener getSelectionListener() {
        return new ISelectionChangedListener() {
            public void selectionChanged(SelectionChangedEvent event) {
                fSelectedKind = fTable.getSelectionIndex();
            }
        };
    }

    protected void configureShell(Shell newShell) {
        super.configureShell(newShell);
        newShell.setText(fTitle);
    }
}
