/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/

package com.ibm.team.build.internal.ui.dialogs;

import java.util.ArrayList;

import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.forms.widgets.Section;

import com.ibm.team.build.internal.ui.BuildPropertyControl;
import com.ibm.team.build.internal.ui.BuildPropertyControl.LayoutSize;
import com.ibm.team.build.ui.dialogs.requestbuild.IRequestBuildSectionFactory;
import com.ibm.team.build.ui.dialogs.requestbuild.RequestBuildSection;
import com.ibm.team.build.ui.dialogs.requestbuild.RequestBuildSectionSite;

/**
 * The build properties section in the request build dialog.
 */
public class PropertiesSection extends RequestBuildSection {

    private IWorkbenchPartSite fPartSite;
    
    /**
     * Factory interface for creating the section.
     */
    public static class Factory implements IRequestBuildSectionFactory {
        public RequestBuildSection createRequestBuildSection(RequestBuildSectionSite site) {
            return new PropertiesSection(site);
        }
    }

    private Composite fPropertiesComposite;

    protected BuildPropertyControl fBuildPropertyControl;

    /**
     * Create the section.
     * 
     * @param site
     *            the section site
     */
    public PropertiesSection(RequestBuildSectionSite site) {
        super(site);
    }

    @Override
    protected String getSectionName() {
        if (getSite().isRebuild() && getSite().getBuildResult() != null) {
            return NLS.bind(BuildUIDialogMessages.RequestBuildDialog_REBUILD_PROPERTIES_LABEL,
                    getSite().getBuildResult().getLabel());
        }
        return BuildUIDialogMessages.RequestBuildDialog_BUILD_PROPERTIES_LABEL;
    }

    @Override
    protected void createSectionContent(Section section) {
        // Dispose any pre existing properties composite. There may be one if
        // the user is clicking Browse to select a different build.
        if (fPropertiesComposite != null && !fPropertiesComposite.isDisposed()) {
            fPropertiesComposite.dispose();
        }

        fPropertiesComposite = new Composite(section, SWT.NONE);
        GridLayoutFactory.fillDefaults().extendedMargins(10, 0, 0, 0).applyTo(fPropertiesComposite);
        section.setClient(fPropertiesComposite);

        fBuildPropertyControl = new BuildPropertyControl(fPropertiesComposite, null, new ArrayList(),
                getSite().getTeamRepository(), true, LayoutSize.COMPACT);

        fBuildPropertyControl.createAndRegisterDefaultContextMenu(fPartSite);
        GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true);
        gridData.heightHint = 250;
        fBuildPropertyControl.getComposite().setLayoutData(gridData);

        fBuildPropertyControl.setBuildProperties(getSite().getBuildDefinition().getProperties());
    }
    
    @Override
    protected boolean grabVertical() {
        return true;
    }
    
    /**
     * Set the workbench part site to register context menu for extension from. 
     * 
     * @param partSite
     *            The workbench part site. May be <code>null</code>. If
     *            <code>null</code>, context menus within the section are not
     *            registered for extension.
     */
    public void setWorkbenchPartSite(IWorkbenchPartSite partSite) {
        fPartSite = partSite;
    }

}
