/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.build.internal.ui.notification;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Job to fetch the names of contributors given their user ids.
 */
public class ContributorFetchJob extends Job {

    private final ITeamRepository fTeamRepository;

    private final Control fControl;

    private final Display fDisplay;

    private final AbstractUserNameRetrievalRunnable fRunnable;

    private String[] fUserIds = new String[0];

    public ContributorFetchJob(ITeamRepository teamRepository, Control control, AbstractUserNameRetrievalRunnable runnable) {
        super(Messages.EditEmailNotificationCriteriaDialog_ContributorsQueryJobName);
        fTeamRepository = teamRepository;
        fControl = control;
        fDisplay = control.getDisplay();
        fRunnable = runnable;
    }

    public synchronized void setUserIds(String[] userIds) {
        fUserIds = userIds;
    }

    private synchronized String[] getUserIds() {
        return fUserIds;
    }

    @Override
    protected IStatus run(IProgressMonitor monitor) {
        String[] userIds = getUserIds();
        final Map<String, String> userNames = new HashMap<String, String>();
        SubMonitor subMonitor = SubMonitor.convert(monitor, userIds.length);

        for (int i = 0; i < userIds.length; i++) {
            if (subMonitor.isCanceled()) {
                return Status.CANCEL_STATUS;
            }

            try {
                userNames.put(userIds[i], fTeamRepository.contributorManager().fetchContributorByUserId(userIds[i],
                        subMonitor.newChild(1)).getName());
            } catch (TeamRepositoryException e) {
                return Status.CANCEL_STATUS;
            }
        }

        if (subMonitor.isCanceled()) {
            return Status.CANCEL_STATUS;
        }

        fDisplay.asyncExec(new Runnable() {
            public void run() {
                if (!fControl.isDisposed()) {
                    fRunnable.setUserNameMap(userNames);
                    fRunnable.run();
                }
            }
        });

        return Status.OK_STATUS;
    }

}