/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

package com.ibm.team.build.internal.ui.notification;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.swt.widgets.Display;

import com.ibm.team.process.client.IClientProcess;
import com.ibm.team.process.client.IProcessItemService;
import com.ibm.team.process.common.IProcessArea;
import com.ibm.team.process.common.IProcessAreaHandle;
import com.ibm.team.process.common.IRole;
import com.ibm.team.repository.client.IItemManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;

/**
 * Job to fetch the available role types
 * 
 * @since 0.8
 */
public class RoleTypesFetchJob extends Job {

    private final ITeamRepository fTeamRepository;

    private final IProcessAreaHandle fProcessAreaHandle;

    private final RoleSelectionDialog fRoleSelectionDialog;

    public RoleTypesFetchJob(ITeamRepository teamRepository, IProcessAreaHandle processAreaHandle,
            RoleSelectionDialog dialog) {
        super(Messages.EditEmailNotificationCriteriaDialog_RoleTypesQueryJobName);
        fTeamRepository = teamRepository;
        fProcessAreaHandle = processAreaHandle;
        fRoleSelectionDialog = dialog;
    }

    protected boolean isRoleSelectionSelectionDialogDisposed() {
        return fRoleSelectionDialog.getShell() == null || fRoleSelectionDialog.getShell().isDisposed();
    }

    protected void asyncExec(Runnable runnable) {
        Display.getDefault().asyncExec(runnable);
    }

    @Override
    protected IStatus run(IProgressMonitor monitor) {
        final List<String> roleDefinitions = new ArrayList<String>();
        try {
            IProcessArea fProcessArea = (IProcessArea) fTeamRepository.itemManager().fetchCompleteItem(
                    fProcessAreaHandle, IItemManager.REFRESH, monitor);

            IProcessItemService service = (IProcessItemService) fTeamRepository.getClientLibrary(IProcessItemService.class);
            IClientProcess clientProcess = service.getClientProcess(fProcessArea, monitor);
            IRole[] availableRoles = clientProcess.getRoles(fProcessArea, monitor);
            for (int i = 0; i < availableRoles.length; i++) {
                roleDefinitions.add(availableRoles[i].getId());
            }

        } catch (TeamRepositoryException e) {
            return new Status(IStatus.ERROR, "com.ibm.team.build.notification.ui", //$NON-NLS-1$
                    Messages.EditEmailNotificationCriteriaDialog_RoleTypesQueryJobError, e);
        }

        asyncExec(new Runnable() {
            public void run() {
                if (isRoleSelectionSelectionDialogDisposed()) {
                    return;
                }
                fRoleSelectionDialog.setFetchingDone();
                fRoleSelectionDialog.setListElements(roleDefinitions.toArray(new String[roleDefinitions.size()]));
            }
        });

        return Status.OK_STATUS;
    }

}